@@ -4,12 +4,15 @@
 XTRACE=$(set +o | grep xtrace)
 set -o xtrace
 
-source $DEST/rally/devstack/lib/rally
+DIR=$(dirname ${BASH_SOURCE[0]})
+source $DIR/lib/rally
 
-if [[ "$1" == "source" ]]; then
-    # Initial source
-    source $TOP_DIR/lib/rally
-elif [[ "$1" == "stack" && "$2" == "install" ]]; then
+MESSAGE="'rally git://git.openstack.org/openstack/rally' devstack plugin is
+ deprecated and will be removed soon! Use 'rally-openstack
+ git://git.openstack.org/openstack/rally-openstack' instead."
+deprecated "$MESSAGE"
+
+if [[ "$1" == "stack" && "$2" == "install" ]]; then
     echo_summary "Installing Rally"
     install_rally
 elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
