@@ -21,6 +21,7 @@ VERBOSE=""
 ASKCONFIRMATION=1
 RECREATEDEST="ask"
 USEVIRTUALENV="yes"
+DEVELOPMENT_MODE="false"
 
 # ansi colors for formatting heredoc
 ESC=$(printf "\e")
@@ -31,8 +32,8 @@ RED="$ESC[0;31m"
 PYTHON2=$(which python || true)
 PYTHON3=$(which python3 || true)
 PYTHON=${PYTHON2:-$PYTHON3}
-BASE_PIP_URL=${BASE_PIP_URL:-"https://pypi.python.org/simple"}
-VIRTUALENV_191_URL="https://raw.github.com/pypa/virtualenv/1.9.1/virtualenv.py"
+VIRTUALENV_VERSION="15.1.0"
+VIRTUALENV_URL="https://raw.github.com/pypa/virtualenv/$VIRTUALENV_VERSION/virtualenv.py"
 
 RALLY_GIT_URL="https://git.openstack.org/openstack/rally"
 RALLY_GIT_BRANCH="master"
@@ -166,27 +167,27 @@ $GREEN  --url                 $NO_COLOR Git repository public URL to download Ra
                          from custom repository.
                          (Default: ${RALLY_GIT_URL}).
                          (Ignored when you are already in git repository).
-$GREEN  --branch              $NO_COLOR Git branch name name or git tag (Rally release) to install.
-                         (Default: latest - master).
-                         (Ignored when you are already in git repository).
+$GREEN  --branch              $NO_COLOR Git branch name, tag (Rally release), commit hash, ref, or other
+                         tree-ish to install. (Default: master)
+                         Ignored when you are already in git repository.
 $GREEN  -f, --overwrite       $NO_COLOR Deprecated. Use -r instead.
 $GREEN  -r, --recreate        $NO_COLOR Remove target directory if it already exist.
                          If neither '-r' nor '-R' is set default behaviour is to ask.
-$GREEN  -R, --no-recreate     $NO_COLOR Do not reemove target directory if it already exist.
+$GREEN  -R, --no-recreate     $NO_COLOR Do not remove target directory if it already exist.
                          If neither '-r' nor '-R' is set default behaviour is to ask.
 $GREEN  -y, --yes             $NO_COLOR Do not ask for confirmation: assume a 'yes' reply
                          to every question.
 $GREEN  -D, --dbtype TYPE     $NO_COLOR Select the database type. TYPE can be one of
-                         'sqlite', 'mysql', 'postgres'.
+                         'sqlite', 'mysql', 'postgresql'.
                          Default: sqlite
 $GREEN  --db-user USER        $NO_COLOR Database user to use. Only used when --dbtype
-                         is either 'mysql' or 'postgres'.
+                         is either 'mysql' or 'postgresql'.
 $GREEN  --db-password PASSWORD$NO_COLOR Password of the database user. Only used when
-                         --dbtype is either 'mysql' or 'postgres'.
+                         --dbtype is either 'mysql' or 'postgresql'.
 $GREEN  --db-host HOST        $NO_COLOR Database host. Only used when --dbtype is
-                         either 'mysql' or 'postgres'
+                         either 'mysql' or 'postgresql'
 $GREEN  --db-name NAME        $NO_COLOR Name of the database. Only used when --dbtype is
-                         either 'mysql' or 'postgres'
+                         either 'mysql' or 'postgresql'
 $GREEN  -p, --python EXE      $NO_COLOR The python interpreter to use. Default: $PYTHON
 $GREEN  --develop             $NO_COLOR Install Rally with editable source code try.
                          (Default: false)
@@ -245,62 +246,54 @@ __EOF__
     fi
 }
 
-have_sw_package () {
-    # instead of guessing which distribution this is, we check for the
-    # package manager name as it basically identifies the distro
-    if have_command dpkg; then
-        (dpkg -l "$1" | egrep -q ^i ) >/dev/null 2>/dev/null
-    elif have_command rpm; then
-        rpm -q "$1" >/dev/null 2>/dev/null
+which_missing_packages () {
+    if [ ! -f bindep.txt ]; then
+        abort $EX_PROTOCOL \
+            "bindep.txt not found. Unable to find missing packages."
     fi
+    require_command "bindep"
+    require_command "lsb_release"
+    echo "$(bindep -b | tr '\n' ' ')"
 }
 
-which_missing_packages () {
-    local missing=''
-    for pkgname in "$@"; do
-        if have_sw_package "$pkgname"; then
-            continue;
-        else
-            missing="$missing $pkgname"
-        fi
-    done
+which_missing_commands () {
+    # These commands are required to run install_rally.sh
+    local missing=""
+    if ! have_command "wget"; then
+        missing="wget"
+    fi
+    if ! have_command "git"; then
+        missing="$missing git"
+    fi
+    if ! have_command "pip"; then
+        missing="$missing python-pip"
+    fi
     echo "$missing"
 }
 
+
 # Download command
 download() {
     wget -nv $VERBOSE --no-check-certificate -O "$@";
 }
 
-download_from_pypi () {
-    local pkg=$1
-    local url=$(download - "$BASE_PIP_URL"/"$pkg"/ | sed -n '/source\/.\/'"$pkg"'.*gz/ { s:.*href="\([^#"]*\)["#].*:\1:g; p; }' | sort | tail -1)
-    if [ -n "$url" ]; then
-        download "$(basename "$url")" "$BASE_PIP_URL"/"$pkg"/"$url"
-    else
-        die $EX_PROTOCOL "Package '$pkg' not found on PyPI!" <<__EOF__
-Unable to download package '$pkg' from PyPI.
-__EOF__
-    fi
-}
-
-install_required_sw () {
-    # instead of guessing which distribution this is, we check for the
-    # package manager name as it basically identifies the distro
-    local missing pkg_manager
+get_pkg_manager () {
     if have_command apt-get; then
         # Debian/Ubuntu
-        missing=$(which_missing_packages build-essential libssl-dev libffi-dev python-dev libxml2-dev libxslt1-dev libpq-dev git wget)
-
         if [ "$ASKCONFIRMATION" -eq 0 ]; then
-            pkg_manager="apt-get install --yes"
+            pkg_manager="apt-get install --yes --force-yes"
         else
             pkg_manager="apt-get install"
         fi
+    elif have_command dnf; then
+        # dnf based RHEL/CentOS/Fedora
+        if [ "$ASKCONFIRMATION" -eq 0 ]; then
+            pkg_manager="dnf install -y"
+        else
+            pkg_manager="dnf install"
+        fi
     elif have_command yum; then
-        # RHEL/CentOS
-        missing=$(which_missing_packages gcc libffi-devel python-devel openssl-devel gmp-devel libxml2-devel libxslt-devel postgresql-devel redhat-rpm-config git wget)
-
+        # yum based RHEL/CentOS/Fedora
         if [ "$ASKCONFIRMATION" -eq 0 ]; then
             pkg_manager="yum install -y"
         else
@@ -308,8 +301,6 @@ install_required_sw () {
         fi
     elif have_command zypper; then
         # SuSE
-        missing=$(which_missing_packages gcc libffi48-devel python-devel openssl-devel gmp-devel libxml2-devel libxslt-devel postgresql93-devel git wget)
-
         if [ "$ASKCONFIRMATION" -eq 0 ]; then
             pkg_manager="zypper -n --no-gpg-checks --non-interactive install --auto-agree-with-licenses"
         else
@@ -319,9 +310,15 @@ install_required_sw () {
         # MacOSX maybe?
         echo "Cannot determine what package manager this system has, so I cannot check if requisite software is installed. I'm proceeding anyway, but you may run into errors later."
     fi
-    if ! have_command pip; then
-        missing="$missing python-pip"
-    fi
+    echo $pkg_manager
+}
+
+install_required_sw () {
+    # instead of guessing which distribution this is, we check for the
+    # package manager name as it basically identifies the distro
+    local missing pkg_manager
+    missing=$1
+    pkg_manager=$(get_pkg_manager)
 
     if [ -n "$missing" ]; then
         cat <<__EOF__
@@ -346,9 +343,9 @@ __EOF__
                     missing=${missing//python-pip/}
                     if ! $pkg_manager python-pip; then
                         if ask_yn "Error installing python-pip. Install from external source?"; then
-                            local pdir=$(mktemp -d)
+                            local pdir=$(mktemp /tmp/tmp.XXXXXXXXXX -d)
                             local getpip="$pdir/get-pip.py"
-                            download "$getpip" https://raw.github.com/pypa/pip/master/contrib/get-pip.py
+                            download "$getpip" https://bootstrap.pypa.io/get-pip.py
                             if ! "$PYTHON" "$getpip"; then
                                 abort $EX_PROTOCOL "Error while installing python-pip from external source."
                             fi
@@ -358,7 +355,7 @@ __EOF__
                         fi
                     fi
                 fi
-                if ! $pkg_manager $missing; then
+                if [ -n "$missing" ] && ! $pkg_manager $missing; then
                     abort $EX_UNAVAILABLE "Error while installing $missing"
                 fi
                 # installation successful
@@ -400,7 +397,7 @@ install_db_connector () {
         mysql)
             pip install pymysql
             ;;
-        postgres)
+        postgresql)
             pip install psycopg2
             ;;
     esac
@@ -419,35 +416,30 @@ __EOF__
     fi
 
     # Use the latest virtualenv that can use `.tar.gz` files
-    VIRTUALENV_DST="$DESTDIR/virtualenv-191.py"
+    VIRTUALENV_DST="$DESTDIR/virtualenv-$VIRTUALENV_VERSION.py"
     mkdir -p "$DESTDIR"
-    download "$VIRTUALENV_DST" "$VIRTUALENV_191_URL"
-    "$PYTHON" "$VIRTUALENV_DST" $VERBOSE -p "$PYTHON" "$DESTDIR"
+    download "$VIRTUALENV_DST" "$VIRTUALENV_URL"
+    "$PYTHON" "$VIRTUALENV_DST" $VERBOSE --no-setuptools --no-pip --no-wheel \
+        -p "$PYTHON" "$DESTDIR"
 
-    . "$DESTDIR"/bin/activate
+    . "$DESTDIR/bin/activate"
 
-    # Recent versions of `pip` insist that setuptools>=0.8 is installed,
-    # because they try to use the "wheel" format for any kind of package.
-    # So we need to update setuptools, or `pip` will error out::
-    #
-    #     Wheel installs require setuptools >= 0.8 for dist-info support.
-    #
-    if pip wheel --help 1>/dev/null 2>/dev/null; then
-        (cd "$DESTDIR" && download_from_pypi setuptools)
-        # setup.py must be called with `python', which will be the
-        # python executable inside the virtualenv, not `$PYTHON',
-        # which is the system python.
-        if ! (cd "$DESTDIR" && tar -xzf setuptools-*.tar.gz && \
-              cd setuptools-* && python setup.py install);
-        then
-            die $EX_SOFTWARE \
-                "Failed to install the latest version of Python 'setuptools'" <<__EOF__
+    download - https://bootstrap.pypa.io/get-pip.py | python -\
+        || die $EX_PROTOCOL \
+        "Error while running get-pip.py" <<__EOF__
 
-The required Python package setuptools could not be installed.
+The required Python package pip could not be installed
+in virtualenv.
+
+__EOF__
+
+    pip install setuptools wheel || die $EX_PROTOCOL \
+        "Error while running 'pip install setuptools wheel'" <<__EOF__
+
+The required Python package setuptools, wheel could not be installed
+in virtualenv.
 
 __EOF__
-        fi
-    fi
 }
 
 setup_rally_configuration () {
@@ -459,11 +451,11 @@ setup_rally_configuration () {
     cp "$SRCDIR"/etc/rally/rally.conf.sample "$ETCDIR"/rally.conf
 
     [ -d "$DBDIR" ] || mkdir -p "$DBDIR"
-    local CONF_TMPFILE=$(mktemp)
+    local CONF_TMPFILE=$(mktemp /tmp/tmp.XXXXXXXXXX)
     sed "s|#connection *=.*|connection = \"$DBCONNSTRING\"|" "$ETCDIR"/rally.conf > "$CONF_TMPFILE"
     cat "$CONF_TMPFILE" > "$ETCDIR"/rally.conf
     rm "$CONF_TMPFILE"
-    rally-manage db recreate
+    rally db recreate
 }
 
 rally_venv () {
@@ -543,7 +535,7 @@ do
             shift
             DBTYPE=$1
             case $DBTYPE in
-                sqlite|mysql|postgres);;
+                sqlite|mysql|postgresql);;
                 *)
                     print_usage | die $EX_USAGE \
                         "An invalid option has been detected."
@@ -645,13 +637,15 @@ __EOF__
     DBAUTH="$DBUSER:$DBPASSWORD@$DBHOST"
     if [ "$DBTYPE" = 'mysql' ]; then
         DBCONNSTRING="$DBTYPE+pymysql://$DBAUTH/$DBNAME"
+    elif  [ "$DBTYPE" = 'postgresql' ]; then
+        DBCONNSTRING="$DBTYPE+psycopg2://$DBAUTH/$DBNAME"
     else
         DBCONNSTRING="$DBTYPE://$DBAUTH/$DBNAME"
     fi
 fi
 
 # check and install prerequisites
-install_required_sw
+install_required_sw "$(which_missing_commands)"
 require_python
 
 
@@ -702,15 +696,14 @@ BASEDIR=$(dirname "$(readlink -e "$0")")
 if [ -d "$BASEDIR"/.git ]
 then
     SOURCEDIR=$BASEDIR
-    pushd $BASEDIR > /dev/null
-    if find . -name '*.py[co]' -exec rm -f {} +
-    then
-        echo "Wiped python compiled files."
-    else
-        echo "Warning! Unable to wipe python compiled files"
-    fi
-
-    popd > /dev/null
+    (
+        cd "$BASEDIR"
+        if find . -name '*.py[co]' -exec rm -f {} +; then
+            echo "Wiped python compiled files."
+        else
+            echo "Warning! Unable to wipe python compiled files"
+        fi
+    )
 else
     if [ "$USEVIRTUALENV" = 'yes' ]
     then
@@ -721,11 +714,14 @@ else
 
     if ! [ -d "$SOURCEDIR"/.git ]
     then
-        echo "Downloading Rally from subversion repository $RALLY_GIT_URL ..."
+        echo "Downloading Rally from git repository $RALLY_GIT_URL ..."
         CURRENT_ACTION="downloading-src"
-        git clone "$RALLY_GIT_URL" -b "$RALLY_GIT_BRANCH" "$SOURCEDIR"
-        if ! [ -d $SOURCEDIR/.git ]
-            then
+        git clone "$RALLY_GIT_URL" "$SOURCEDIR"
+        (
+            cd "$SOURCEDIR"
+            git checkout "$RALLY_GIT_BRANCH"
+        )
+        if ! [ -d "$SOURCEDIR"/.git ]; then
             abort $EX_CANTCREAT "Unable to download git repository"
         fi
         CURRENT_ACTION="src-downloaded"
@@ -737,19 +733,22 @@ install_db_connector
 # Install rally
 cd "$SOURCEDIR"
 # Get latest available pip and reset shell cache
-pip install -i $BASE_PIP_URL -U 'pip'
+pip install -U 'pip!=8'
 hash -r
 
 # Install dependencies
-pip install -i $BASE_PIP_URL pbr 'tox<=1.6.1'
+pip install pbr 'tox<=1.6.1' bindep
+
+# Install binary dependencies
+install_required_sw "$(which_missing_packages)"
+
 # Uninstall possible previous version
 pip uninstall -y rally || true
 # Install rally
-if [ $DEVELOPMENT_MODE ]
-then
-    pip install -i $BASE_PIP_URL -e .
+if [ "$DEVELOPMENT_MODE" = "true" ]; then
+    pip install --constraint ./upper-constraints.txt --editable .
 else
-    pip install -i $BASE_PIP_URL .
+    pip install --constraint ./upper-constraints.txt .
 fi
 
 cd "$ORIG_WD"
@@ -765,16 +764,14 @@ __EOF__
 
     setup_rally_configuration "$SOURCEDIR"
 
-    if ! [ $DEVELOPMENT_MODE ]
-    then
+    if ! [ "$DEVELOPMENT_MODE" = "true" ]; then
         SAMPLESDIR=$VENVDIR/samples
-        mkdir -p $SAMPLESDIR
-        cp -r $SOURCEDIR/samples/* $SAMPLESDIR/
-    else
-        SAMPLESDIR=$SOURCEDIR/samples
+        mkdir -p "$SAMPLESDIR"
+        cp -r "$SOURCEDIR"/samples/* "$SAMPLESDIR"/
     fi
-    mkdir -p $VENVDIR/etc/bash_completion.d
-    install $SOURCEDIR/etc/rally.bash_completion $VENVDIR/etc/bash_completion.d/
+    mkdir -p "$VENVDIR"/etc/bash_completion.d
+    install "$SOURCEDIR"/etc/rally.bash_completion \
+        "$VENVDIR"/etc/bash_completion.d/
 
     cat <<__EOF__
 $GREEN==============================
@@ -801,15 +798,13 @@ __EOF__
 else
     setup_rally_configuration "$SOURCEDIR"
 
-    if ! [ $DEVELOPMENT_MODE ]
-    then
+    if ! [ "$DEVELOPMENT_MODE" = "true" ]; then
         SAMPLESDIR=/usr/share/rally/samples
-        mkdir -p $SAMPLESDIR
-        cp -r $SOURCEDIR/samples/* $SAMPLESDIR/
-    else
-        SAMPLESDIR=$SOURCEDIR/samples
+        mkdir -p "$SAMPLESDIR"
+        cp -r "$SOURCEDIR"/samples/* "$SAMPLESDIR"/
     fi
-    ln -s /usr/local/etc/bash_completion.d/rally.bash_completion /etc/bash_completion.d/ 2> /dev/null || true
+    ln -s /usr/local/etc/bash_completion.d/rally.bash_completion \
+        /etc/bash_completion.d/ 2> /dev/null || true
     if [ -f "${DBFILE}" ]; then
         chmod 777 "$DBFILE"
     fi
