@@ -17,27 +17,23 @@ from __future__ import print_function
 
 import argparse
 import inspect
+import json
 import os
 import sys
+import textwrap
 import warnings
 
-import decorator
 import jsonschema
-from oslo_config import cfg
-from oslo_utils import encodeutils
 import prettytable
 import six
 import sqlalchemy.exc
 
-from rally.common.i18n import _
+from rally import api
+from rally.common import cfg
 from rally.common import logging
-from rally.common.plugin import discover
 from rally.common.plugin import info
-from rally.common import version
 from rally import exceptions
-
-
-CONFIG_SEARCH_PATHS = [sys.prefix + "/etc/rally", "~/.rally", "/etc/rally"]
+from rally.utils import encodeutils
 
 
 CONF = cfg.CONF
@@ -48,21 +44,11 @@ LOG = logging.getLogger(__name__)
 MARGIN = 3
 
 
-def find_config_files(path_list):
-    for path in path_list:
-        abspath = os.path.abspath(os.path.expanduser(path))
-        confname = abspath + "/rally.conf"
-        if os.path.isfile(confname):
-            return [confname]
-
-    return None
-
-
 class MissingArgs(Exception):
     """Supplied arguments are not sufficient for calling a function."""
     def __init__(self, missing):
         self.missing = missing
-        msg = _("Missing arguments: %s") % ", ".join(missing)
+        msg = "Missing arguments: %s" % ", ".join(missing)
         super(MissingArgs, self).__init__(msg)
 
 
@@ -79,7 +65,7 @@ def validate_args(fn, *args, **kwargs):
     MissingArgs: Missing argument(s): b, d
 
     :param fn: the function to check
-    :param arg: the positional arguments supplied
+    :param args: the positional arguments supplied
     :param kwargs: the keyword arguments supplied
     """
     argspec = inspect.getargspec(fn)
@@ -87,20 +73,18 @@ def validate_args(fn, *args, **kwargs):
     num_defaults = len(argspec.defaults or [])
     required_args = argspec.args[:len(argspec.args) - num_defaults]
 
-    def isbound(method):
-        return getattr(method, "__self__", None) is not None
-
-    if isbound(fn):
+    if getattr(fn, "__self__", None):
         required_args.pop(0)
 
-    missing = [arg for arg in required_args if arg not in kwargs]
-    missing = missing[len(args):]
+    missing_required_args = required_args[len(args):]
+    missing = [arg for arg in missing_required_args if arg not in kwargs]
     if missing:
         raise MissingArgs(missing)
 
 
 def print_list(objs, fields, formatters=None, sortby_index=0,
                mixed_case_fields=None, field_labels=None,
+               normalize_field_names=False,
                table_label=None, print_header=True, print_border=True,
                out=sys.stdout):
     """Print a list or objects as a table, one row per object.
@@ -113,18 +97,22 @@ def print_list(objs, fields, formatters=None, sortby_index=0,
         have mixed case names (e.g., 'serverId')
     :param field_labels: Labels to use in the heading of the table, default to
         fields.
+    :param normalize_field_names: If True, field names will be transformed,
+        e.g. "Field Name" -> "field_name", otherwise they will be used
+        unchanged.
     :param table_label: Label to use as header for the whole table.
     :param print_header: print table header.
     :param print_border: print table border.
     :param out: stream to write output to.
+
     """
     formatters = formatters or {}
     mixed_case_fields = mixed_case_fields or []
     field_labels = field_labels or fields
     if len(field_labels) != len(fields):
-        raise ValueError(_("Field labels list %(labels)s has different number "
-                           "of elements than fields list %(fields)s"),
-                         {"labels": field_labels, "fields": fields})
+        raise ValueError("Field labels list %(labels)s has different number of"
+                         " elements than fields list %(fields)s"
+                         % {"labels": field_labels, "fields": fields})
 
     if sortby_index is None:
         kwargs = {}
@@ -138,14 +126,18 @@ def print_list(objs, fields, formatters=None, sortby_index=0,
         for field in fields:
             if field in formatters:
                 row.append(formatters[field](o))
-            elif type(o) == dict:
-                row.append(o[field])
             else:
-                if field in mixed_case_fields:
-                    field_name = field.replace(" ", "_")
+                field_name = field
+
+                if normalize_field_names:
+                    if field_name not in mixed_case_fields:
+                        field_name = field_name.lower()
+                    field_name = field_name.replace(" ", "_").replace("-", "_")
+
+                if isinstance(o, dict):
+                    data = o.get(field_name, "")
                 else:
-                    field_name = field.lower().replace(" ", "_")
-                data = getattr(o, field_name, "")
+                    data = getattr(o, field_name, "")
                 row.append(data)
         pt.add_row(row)
 
@@ -175,6 +167,97 @@ def print_list(objs, fields, formatters=None, sortby_index=0,
         out.write(encodeutils.safe_encode(table_body))
 
 
+def print_dict(obj, fields=None, formatters=None, mixed_case_fields=False,
+               normalize_field_names=False, property_label="Property",
+               value_label="Value", table_label=None, print_header=True,
+               print_border=True, wrap=0, out=sys.stdout):
+    """Print dict as a table.
+
+    :param obj: dict to print
+    :param fields: `dict` of keys to print from d. Defaults to all keys
+    :param formatters: `dict` of callables for field formatting
+    :param mixed_case_fields: fields corresponding to object attributes that
+        have mixed case names (e.g., 'serverId')
+    :param normalize_field_names: If True, field names will be transformed,
+        e.g. "Field Name" -> "field_name", otherwise they will be used
+        unchanged.
+    :param property_label: label of "property" column
+    :param value_label: label of "value" column
+    :param table_label: Label to use as header for the whole table.
+    :param print_header: print table header.
+    :param print_border: print table border.
+    :param out: stream to write output to.
+    """
+    formatters = formatters or {}
+    mixed_case_fields = mixed_case_fields or []
+    if not fields:
+        if isinstance(obj, dict):
+            fields = sorted(obj.keys())
+        else:
+            fields = [name for name in dir(obj)
+                      if (not name.startswith("_") and
+                          not callable(getattr(obj, name)))]
+
+    pt = prettytable.PrettyTable([property_label, value_label], caching=False)
+    pt.align = "l"
+    for field_name in fields:
+        if field_name in formatters:
+            data = formatters[field_name](obj)
+        else:
+            field = field_name
+            if normalize_field_names:
+                if field not in mixed_case_fields:
+                    field = field_name.lower()
+                field = field.replace(" ", "_").replace("-", "_")
+
+            if isinstance(obj, dict):
+                data = obj.get(field, "")
+            else:
+                data = getattr(obj, field, "")
+
+        # convert dict to str to check length
+        if isinstance(data, (dict, list)):
+            data = json.dumps(data)
+        if wrap > 0:
+            data = textwrap.fill(six.text_type(data), wrap)
+        # if value has a newline, add in multiple rows
+        # e.g. fault with stacktrace
+        if (data and
+                isinstance(data, six.string_types) and
+                (r"\n" in data or "\r" in data)):
+            # "\r" would break the table, so remove it.
+            if "\r" in data:
+                data = data.replace("\r", "")
+            lines = data.strip().split(r"\n")
+            col1 = field_name
+            for line in lines:
+                pt.add_row([col1, line])
+                col1 = ""
+        else:
+            if data is None:
+                data = "-"
+            pt.add_row([field_name, data])
+
+    table_body = pt.get_string(header=print_header,
+                               border=print_border) + "\n"
+
+    table_header = ""
+
+    if table_label:
+        table_width = table_body.index("\n")
+        table_header = make_table_header(table_label, table_width)
+        table_header += "\n"
+
+    if six.PY3:
+        if table_header:
+            out.write(encodeutils.safe_encode(table_header).decode())
+        out.write(encodeutils.safe_encode(table_body).decode())
+    else:
+        if table_header:
+            out.write(encodeutils.safe_encode(table_header))
+        out.write(encodeutils.safe_encode(table_body))
+
+
 def make_table_header(table_label, table_width,
                       junction_char="+", horizontal_char="-",
                       vertical_char="|"):
@@ -191,8 +274,8 @@ def make_table_header(table_label, table_width,
     """
 
     if len(table_label) >= (table_width - 2):
-        raise ValueError(_("Table header %s is longer than total"
-                           "width of the table."))
+        raise ValueError(
+            "Table header %s is longer than total width of the table.")
 
     label_and_space_width = table_width - len(table_label) - 2
     padding = 0 if label_and_space_width % 2 == 0 else 1
@@ -222,7 +305,7 @@ def make_header(text, size=80, symbol="-"):
     :param symbol: What symbol to use to create header
     """
     header = symbol * size + "\n"
-    header += " %s\n" % text
+    header += "%s\n" % text
     header += symbol * size + "\n"
     return header
 
@@ -232,23 +315,6 @@ def suppress_warnings(f):
     return f
 
 
-@decorator.decorator
-def process_keystone_exc(f, *args, **kwargs):
-    from keystoneclient import exceptions as keystone_exc
-
-    try:
-        return f(*args, **kwargs)
-    except keystone_exc.Unauthorized as e:
-        print(_("User credentials are wrong! \n%s") % e)
-        return 1
-    except keystone_exc.AuthorizationFailure as e:
-        print(_("Failed to authorize! \n%s") % e)
-        return 1
-    except keystone_exc.ConnectionRefused as e:
-        print(_("Rally can't reach the Keystone service! \n%s") % e)
-        return 1
-
-
 class CategoryParser(argparse.ArgumentParser):
 
     """Customized arguments parser
@@ -284,6 +350,13 @@ class CategoryParser(argparse.ArgumentParser):
 
     def error(self, message):
         self.print_help(sys.stderr)
+        if message.startswith("argument") and message.endswith("is required"):
+            # NOTE(pirsriva) Argparse will currently raise an error
+            # message for only 1 missing argument at a time i.e. in the
+            # error message it WILL NOT LIST ALL the missing arguments
+            # at once INSTEAD only 1 missing argument at a time
+            missing_arg = message.split()[1]
+            print("Missing argument:\n%s" % missing_arg)
         sys.exit(2)
 
 
@@ -296,7 +369,7 @@ def pretty_float_formatter(field, ndigits=None):
     :returns: field formatter function
     """
     def _formatter(obj):
-        value = obj[field] if type(obj) == dict else getattr(obj, field)
+        value = obj[field] if isinstance(obj, dict) else getattr(obj, field)
         if type(value) in (int, float):
             if ndigits:
                 return round(value, ndigits)
@@ -332,20 +405,41 @@ def deprecated_args(*args, **kwargs):
         if "release" not in kwargs:
             raise ValueError("'release' is required keyword argument of "
                              "'deprecated_args' decorator.")
-        func.__dict__.setdefault("args", []).insert(0, (args, kwargs))
-        func.__dict__.setdefault("deprecated_args", [])
-        func.deprecated_args.append(args[0])
+        release = kwargs.pop("release")
+        alternative = kwargs.pop("alternative", None)
 
-        help_msg = "[Deprecated since Rally %s] " % kwargs.pop("release")
-        if "alternative" in kwargs:
-            help_msg += "Use '%s' instead. " % kwargs.pop("alternative")
+        help_msg = "[Deprecated since Rally %s] " % release
+        if alternative:
+            help_msg += "Use '%s' instead. " % alternative
         if "help" in kwargs:
             help_msg += kwargs["help"]
         kwargs["help"] = help_msg
+
+        func.__dict__.setdefault("args", []).insert(0, (args, kwargs))
+        func.__dict__.setdefault("deprecated_args", {})
+        func.deprecated_args[args[0]] = (release, alternative)
         return func
     return _decorator
 
 
+def help_group(uuid):
+    """Label cli method with specific group.
+
+    Joining methods by groups allows to compose more user-friendly help
+    messages in CLI.
+
+    :param uuid: Name of group to find common methods. It will be used for
+        sorting groups in help message, so you can start uuid with
+        some number (i.e "1_launcher", "2_management") to put groups in proper
+        order. Note: default group had "0" uuid.
+    """
+
+    def wrapper(func):
+        func.help_group = uuid
+        return func
+    return wrapper
+
+
 def _methods_of(cls):
     """Get all callable methods of a class that don't start with underscore.
 
@@ -357,6 +451,20 @@ def _methods_of(cls):
     all_methods = inspect.getmembers(
         cls, predicate=lambda x: inspect.ismethod(x) or inspect.isfunction(x))
     methods = [m for m in all_methods if not m[0].startswith("_")]
+
+    help_groups = {}
+    for m in methods:
+        group = getattr(m[1], "help_group", "0")
+        help_groups.setdefault(group, []).append(m)
+
+    if len(help_groups) > 1:
+        # we should sort methods by groups
+        methods = []
+        for group in sorted(help_groups.items(), key=lambda x: x[0]):
+            if methods:
+                # None -> empty line between groups
+                methods.append((None, None))
+            methods.extend(group[1])
     return methods
 
 
@@ -370,10 +478,13 @@ def _compose_category_description(category):
         description = doc.strip()
     if descr_pairs:
         description += "\n\nCommands:\n"
-        sublen = lambda item: len(item[0])
+        sublen = lambda item: len(item[0]) if item[0] else 0
         first_column_len = max(map(sublen, descr_pairs)) + MARGIN
         for item in descr_pairs:
-            name = getattr(item[1], "alias", item[0])
+            if item[0] is None:
+                description += "\n"
+                continue
+            name = getattr(item[1], "alias", item[0].replace("_", "-"))
             if item[1].__doc__:
                 doc = info.parse_docstring(
                     item[1].__doc__)["short_description"]
@@ -420,6 +531,9 @@ def _add_command_parsers(categories, subparsers):
         category_subparsers = parser.add_subparsers(dest="action")
 
         for method_name, method in _methods_of(command_object):
+            if method is None:
+                continue
+            method_name = method_name.replace("_", "-")
             descr = _compose_action_description(method)
             parser = category_subparsers.add_parser(
                 getattr(method, "alias", method_name),
@@ -444,10 +558,13 @@ def validate_deprecated_args(argv, fn):
     if (len(argv) > 3
        and (argv[2] == fn.__name__)
        and getattr(fn, "deprecated_args", None)):
-        for item in fn.deprecated_args:
+        for item, details in fn.deprecated_args.items():
             if item in argv[3:]:
-                LOG.warning("Deprecated argument %s for %s." % (item,
-                                                                fn.__name__))
+                msg = ("The argument `%s` is deprecated since Rally %s." %
+                       (item, details[0]))
+                if details[1]:
+                    msg += " Use `%s` instead." % details[1]
+                LOG.warning(msg)
 
 
 def run(argv, categories):
@@ -469,44 +586,33 @@ def run(argv, categories):
                                           "RALLY_PLUGIN_PATHS"),
                                       help=help_msg))
 
+    # NOTE(andreykurilin): this dirty hack is done to unblock the gates.
+    #   Currently, we are using oslo.config for CLI purpose (don't do this!)
+    #   and it makes the things too complicated.
+    #   To discover which CLI method can be affected by warnings and which not
+    #   (based on suppress_warnings decorator) we need to obtain a desired
+    #   CLI method. It can be done only after initialization of oslo_config
+    #   which is located in rally.api.API init method.
+    #   Initialization of rally.api.API can produce a warning (for example,
+    #   from pymysql), so suppressing of warnings later will not work in such
+    #   case (it is what actually had happened now in our CI with the latest
+    #   release of PyMySQL).
+    #
+    # https://bitbucket.org/zzzeek/sqlalchemy/issues/4120/mysql-5720-warns-on-tx_isolation
     try:
-        CONF(argv[1:], project="rally", version=version.version_string(),
-             default_config_files=find_config_files(CONFIG_SEARCH_PATHS))
-        logging.setup("rally")
-        if not CONF.get("log_config_append"):
-            # The below two lines are to disable noise from request module. The
-            # standard way should be we make such lots of settings on the root
-            # rally. However current oslo codes doesn't support such interface.
-            # So I choose to use a 'hacking' way to avoid INFO logs from
-            # request module where user didn't give specific log configuration.
-            # And we could remove this hacking after oslo.log has such
-            # interface.
-            LOG.debug("INFO logs from urllib3 and requests module are hide.")
-            requests_log = logging.getLogger("requests").logger
-            requests_log.setLevel(logging.WARNING)
-            urllib3_log = logging.getLogger("urllib3").logger
-            urllib3_log.setLevel(logging.WARNING)
+        import pymysql
+        warnings.filterwarnings("ignore", category=pymysql.Warning)
+    except ImportError:
+        pass
 
-            # NOTE(wtakase): This is for suppressing boto error logging.
-            LOG.debug("ERROR log from boto module is hide.")
-            boto_log = logging.getLogger("boto").logger
-            boto_log.setLevel(logging.CRITICAL)
-
-    except cfg.ConfigFilesNotFoundError:
-        cfgfile = CONF.config_file[-1] if CONF.config_file else None
-        if cfgfile and not os.access(cfgfile, os.R_OK):
-            st = os.stat(cfgfile)
-            print(_("Could not read %s. Re-running with sudo") % cfgfile)
-            try:
-                os.execvp("sudo", ["sudo", "-u", "#%s" % st.st_uid] + sys.argv)
-            except Exception:
-                print(_("sudo failed, continuing as if nothing happened"))
-
-        print(_("Please re-run %s as root.") % argv[0])
+    try:
+        rapi = api.API(config_args=argv[1:], skip_db_check=True)
+    except exceptions.RallyException as e:
+        print(e)
         return(2)
 
     if CONF.category.name == "version":
-        print(version.version_string())
+        print(CONF.version)
         return(0)
 
     if CONF.category.name == "bash-completion":
@@ -516,6 +622,8 @@ def run(argv, categories):
     fn = CONF.category.action_fn
     fn_args = [encodeutils.safe_decode(arg)
                for arg in CONF.category.action_args]
+    # api instance always is the first argument
+    fn_args.insert(0, rapi)
     fn_kwargs = {}
     for k in CONF.category.action_kwargs:
         v = getattr(CONF.category, "action_kwarg_" + k)
@@ -544,35 +652,37 @@ def run(argv, categories):
         return(1)
 
     try:
-        for path in CONF.plugin_paths or []:
-            discover.load_plugins(path)
-
         validate_deprecated_args(argv, fn)
 
+        # skip db check for db and plugin commands
+        if CONF.category.name not in ("db", "plugin"):
+            rapi.check_db_revision()
+
         if getattr(fn, "_suppress_warnings", False):
             with warnings.catch_warnings():
                 warnings.simplefilter("ignore")
                 ret = fn(*fn_args, **fn_kwargs)
         else:
             ret = fn(*fn_args, **fn_kwargs)
-        return(ret)
+        return ret
 
-    except (IOError, TypeError, ValueError, exceptions.DeploymentNotFound,
-            exceptions.TaskNotFound, jsonschema.ValidationError) as e:
+    except (IOError, TypeError, ValueError,
+            exceptions.RallyException, jsonschema.ValidationError) as e:
         if logging.is_debug():
-            LOG.exception(e)
-        print(e)
-        return 1
+            LOG.exception("Unexpected exception in CLI")
+        else:
+            print(e)
+        return getattr(e, "error_code", 1)
     except sqlalchemy.exc.OperationalError as e:
         if logging.is_debug():
-            LOG.exception(e)
+            LOG.exception("Something went wrong with database")
         print(e)
         print("Looks like Rally can't connect to its DB.")
-        print("Make a sure that connection string in rally.conf is proper:")
+        print("Make sure that connection string in rally.conf is proper:")
         print(CONF.database.connection)
         return 1
     except Exception:
-        print(_("Command failed, please check log for more info"))
+        print("Command failed, please check log for more info")
         raise
 
 
@@ -631,7 +741,9 @@ complete -o filenames -F _rally rally
     completion = []
     for category, cmds in main.categories.items():
         for name, command in _methods_of(cmds):
-            command_name = getattr(command, "alias", name)
+            if name is None:
+                continue
+            command_name = getattr(command, "alias", name.replace("_", "-"))
             args_list = []
             for arg in getattr(command, "args", []):
                 if getattr(command, "deprecated_args", []):
