@@ -26,40 +26,47 @@ class PluginCommands(object):
 
     @staticmethod
     def _print_plugins_list(plugin_list):
-        rows = [utils.Struct(name=f.get_name(),
-                             namespace=f.get_namespace(),
-                             title=f.get_info()["title"])
-                for f in plugin_list]
+        formatters = {
+            "Name": lambda p: p.get_name(),
+            "Platform": lambda p: p.get_platform(),
+            "Title": lambda p: p.get_info()["title"],
+            "Plugin base": lambda p: p._get_base().__name__
+        }
 
-        cliutils.print_list(rows, fields=["name", "namespace", "title"])
+        cliutils.print_list(plugin_list, formatters=formatters,
+                            normalize_field_names=True,
+                            fields=["Plugin base", "Name", "Platform",
+                                    "Title"])
 
     @cliutils.args("--name", dest="name", type=str,
                    help="Plugin name.")
-    @cliutils.args("--namespace", dest="namespace", type=str,
-                   help="Plugin namespace.")
+    @cliutils.args("--platform", dest="platform", type=str,
+                   help="Plugin platform.")
+    @cliutils.deprecated_args("--namespace", dest="platform",
+                              release="0.10.0", alternative="--platform")
     @plugins.ensure_plugins_are_loaded
-    def show(self, name, namespace=None):
+    def show(self, api, name, platform=None):
         """Show detailed information about a Rally plugin."""
         name_lw = name.lower()
-        all_plugins = plugin.Plugin.get_all(namespace=namespace)
+        all_plugins = plugin.Plugin.get_all(platform=platform)
         found = [p for p in all_plugins if name_lw in p.get_name().lower()]
         exact_match = [p for p in found if name_lw == p.get_name().lower()]
 
         if not found:
-            if namespace:
+            if platform:
                 print(
-                    "There is no plugin: %(name)s in %(namespace)s namespace"
-                    % {"name": name, "namespace": namespace}
+                    "Plugin %(name)s@%(platform)s not found"
+                    % {"name": name, "platform": platform}
                 )
             else:
-                print("There is no plugin: %s" % name)
+                print("Plugin %s not found at any platform" % name)
 
         elif len(found) == 1 or exact_match:
             plugin_ = found[0] if len(found) == 1 else exact_match[0]
             plugin_info = plugin_.get_info()
             print(cliutils.make_header(plugin_info["title"]))
             print("NAME\n\t%s" % plugin_info["name"])
-            print("NAMESPACE\n\t%s" % plugin_info["namespace"])
+            print("PLATFORM\n\t%s" % plugin_info["platform"])
             print("MODULE\n\t%s" % plugin_info["module"])
             if plugin_info["description"]:
                 print("DESCRIPTION\n\t", end="")
@@ -67,31 +74,42 @@ class PluginCommands(object):
             if plugin_info["parameters"]:
                 print("PARAMETERS")
                 rows = [utils.Struct(name=p["name"],
-                                     description="%s\n" % p["doc"])
+                                     description=p["doc"])
                         for p in plugin_info["parameters"]]
-                cliutils.print_list(rows, fields=["name", "description"])
+                cliutils.print_list(rows, fields=["name", "description"],
+                                    sortby_index=None)
         else:
             print("Multiple plugins found:")
             self._print_plugins_list(found)
 
-    @cliutils.args("--name", dest="name", type=str,
-                   help="List only plugins that match the given name.")
     @cliutils.args(
-        "--namespace", dest="namespace", type=str,
-        help="List only plugins that are in the specified namespace.")
+        "--name", dest="name", type=str,
+        help="List only plugins that match the given name.")
+    @cliutils.args(
+        "--platform", dest="platform", type=str,
+        help="List only plugins that are in the specified platform.")
+    @cliutils.deprecated_args("--namespace", dest="platform",
+                              release="0.10.0", alternative="--platform")
+    @cliutils.args(
+        "--plugin-base", dest="base_cls", type=str,
+        help="Plugin base class.")
     @plugins.ensure_plugins_are_loaded
-    def list(self, name=None, namespace=None):
-        """List all Rally plugins that match name and namespace."""
-        all_plugins = plugin.Plugin.get_all(namespace=namespace)
+    def list(self, api, name=None, platform=None, base_cls=None):
+        """List all Rally plugins that match name and platform."""
+        all_plugins = plugin.Plugin.get_all(platform=platform)
         matched = all_plugins
         if name:
             name_lw = name.lower()
             matched = [p for p in all_plugins
                        if name_lw in p.get_name().lower()]
 
+        if base_cls:
+            matched = [p for p in matched
+                       if p._get_base().__name__ == base_cls]
+
         if not all_plugins:
-            print("There is no plugin namespace: %s" % namespace)
+            print("Platform %s not found" % platform)
         elif not matched:
-            print("There is no plugin: %s" % name)
+            print("Plugin %s not found" % name)
         else:
             self._print_plugins_list(matched)
