@@ -13,24 +13,26 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-""" CLI interface for Rally. """
+"""CLI interface for Rally."""
 
 from __future__ import print_function
 
 import sys
 
 from rally.cli import cliutils
+from rally.cli.commands import db
 from rally.cli.commands import deployment
+from rally.cli.commands import env
 from rally.cli.commands import plugin
-from rally.cli.commands import show
 from rally.cli.commands import task
 from rally.cli.commands import verify
 
 
 categories = {
+    "db": db.DBCommands,
+    "env": env.EnvCommands,
     "deployment": deployment.DeploymentCommands,
     "plugin": plugin.PluginCommands,
-    "show": show.ShowCommands,
     "task": task.TaskCommands,
     "verify": verify.VerifyCommands
 }
@@ -39,5 +41,6 @@ categories = {
 def main():
     return cliutils.run(sys.argv, categories)
 
+
 if __name__ == "__main__":
     sys.exit(main())
