@@ -15,20 +15,26 @@
 
 import bisect
 import collections
+import copy
 import ctypes
 import heapq
 import inspect
 import multiprocessing
+import os
 import random
 import re
+import shutil
 import string
 import sys
+import tempfile
 import time
+import uuid
 
 from six import moves
 
 from rally.common import logging
 from rally import exceptions
+from rally.utils import strutils
 
 LOG = logging.getLogger(__name__)
 
@@ -41,7 +47,7 @@ class ImmutableMixin(object):
 
     def __setattr__(self, key, value):
         if self._inited:
-            raise exceptions.ImmutableException()
+            raise AttributeError("This object is immutable.")
         super(ImmutableMixin, self).__setattr__(key, value)
 
 
@@ -77,6 +83,8 @@ class StdErrCapture(object):
 
 
 class Timer(object):
+    """Timer based on context manager interface."""
+
     def __enter__(self):
         self.error = None
         self.start = time.time()
@@ -85,25 +93,38 @@ class Timer(object):
     def timestamp(self):
         return self.start
 
+    def finish_timestamp(self):
+        return self.finish
+
     def __exit__(self, type, value, tb):
         self.finish = time.time()
         if type:
             self.error = (type, value, tb)
 
-    def duration(self):
-        return self.finish - self.start
+    def duration(self, fmt=False):
+        duration = self.finish - self.start
+        if not fmt:
+            return duration
+        if duration > 60:
+            return "%.2f min" % (duration / 60)
+        if duration > 0.1:
+            return "%.2f sec" % duration
+        return "%.2f msec" % (duration * 1000)
 
 
 class Struct(object):
     def __init__(self, **entries):
         self.__dict__.update(entries)
 
+    def __getitem__(self, item, default=None):
+        return getattr(self, item, default)
+
 
 class RAMInt(object):
     """Share RAM integer, for IPC.
 
     This class represents iterable which refers directly to an integer value
-    stored in RAM. Being a true system-level singletone, this allows safely
+    stored in RAM. Being a true system-level singleton, this allows safely
     share integer among processes and threads.
     """
 
@@ -171,6 +192,8 @@ def first_index(lst, predicate):
     return None
 
 
+@logging.log_deprecated(message="Its not used elsewhere in Rally already.",
+                        rally_version="0.4.1")
 def distance(s1, s2):
     """Computes the edit distance between two strings.
 
@@ -248,6 +271,14 @@ class RandomNameGeneratorMixin(object):
     RESOURCE_NAME_FORMAT = "rally_XXXXXXXX_XXXXXXXX"
     RESOURCE_NAME_ALLOWED_CHARACTERS = string.ascii_letters + string.digits
 
+    @classmethod
+    def _get_resource_name_format(cls):
+        return cls.RESOURCE_NAME_FORMAT
+
+    @classmethod
+    def _get_resource_name_allowed_characters(cls):
+        return cls.RESOURCE_NAME_ALLOWED_CHARACTERS
+
     @classmethod
     def _generate_random_part(cls, length):
         """Generate a random string.
@@ -257,8 +288,9 @@ class RandomNameGeneratorMixin(object):
                   containing only characters from
                   cls.RESOURCE_NAME_ALLOWED_CHARACTERS
         """
-        return "".join(random.choice(cls.RESOURCE_NAME_ALLOWED_CHARACTERS)
-                       for i in range(length))
+        return "".join(
+            random.choice(cls._get_resource_name_allowed_characters())
+            for i in range(length))
 
     @classmethod
     def _generate_task_id_part(cls, task_id, length):
@@ -268,16 +300,16 @@ class RandomNameGeneratorMixin(object):
 
         if len(task_id_part) < length:
             LOG.debug("Task ID %(task_id)s cannot be included in a random "
-                      "name because it is too short. Format: %(format)s" %
-                      {"task_id": task_id,
-                       "format": cls.RESOURCE_NAME_FORMAT})
-        elif any(char not in cls.RESOURCE_NAME_ALLOWED_CHARACTERS
+                      "name because it is too short. Format: %(format)s"
+                      % {"task_id": task_id,
+                         "format": cls._get_resource_name_format()})
+        elif any(char not in cls._get_resource_name_allowed_characters()
                  for char in task_id_part):
             LOG.debug("Task ID %(task_id)s cannot be included in a random "
                       "name because it includes disallowed characters. "
-                      "Allowed characters are: %(chars)s" %
-                      {"task_id": task_id,
-                       "chars": cls.RESOURCE_NAME_ALLOWED_CHARACTERS})
+                      "Allowed characters are: %(chars)s"
+                      % {"task_id": task_id,
+                         "chars": cls._get_resource_name_allowed_characters()})
         else:
             return task_id_part
 
@@ -293,6 +325,12 @@ class RandomNameGeneratorMixin(object):
         finally:
             random.seed()
 
+    def get_owner_id(self):
+        if hasattr(self, "task"):
+            return self.task["uuid"]
+        elif hasattr(self, "verification"):
+            return self.verification["uuid"]
+
     def generate_random_name(self):
         """Generate pseudo-random resource name for scenarios.
 
@@ -307,16 +345,13 @@ class RandomNameGeneratorMixin(object):
 
         :returns: str, pseudo-random name
         """
-        if hasattr(self, "task"):
-            task_id = self.task["uuid"]
-        elif hasattr(self, "verification"):
-            task_id = self.verification["uuid"]
+        task_id = self.get_owner_id()
 
         match = self._resource_name_placeholder_re.match(
-            self.RESOURCE_NAME_FORMAT)
+            self._get_resource_name_format())
         if match is None:
             raise ValueError("%s is not a valid resource name format" %
-                             self.RESOURCE_NAME_FORMAT)
+                             self._get_resource_name_format())
         parts = match.groupdict()
         return "".join([
             parts["prefix"],
@@ -341,13 +376,13 @@ class RandomNameGeneratorMixin(object):
         :returns: bool
         """
         match = cls._resource_name_placeholder_re.match(
-            cls.RESOURCE_NAME_FORMAT)
+            cls._get_resource_name_format())
         parts = match.groupdict()
         subst = {
             "prefix": re.escape(parts["prefix"]),
             "sep": re.escape(parts["sep"]),
             "suffix": re.escape(parts["suffix"]),
-            "chars": re.escape(cls.RESOURCE_NAME_ALLOWED_CHARACTERS),
+            "chars": re.escape(cls._get_resource_name_allowed_characters()),
             "rand_length": len(parts["rand"])}
         if task_id:
             subst["task_id"] = cls._generate_task_id_part(task_id,
@@ -383,13 +418,34 @@ def name_matches_object(name, *objects, **kwargs):
     """
     unique_rng_options = {}
     for obj in objects:
-        key = (obj.RESOURCE_NAME_FORMAT, obj.RESOURCE_NAME_ALLOWED_CHARACTERS)
+        key = (obj._get_resource_name_format(),
+               obj._get_resource_name_allowed_characters())
         if key not in unique_rng_options:
             unique_rng_options[key] = obj
     return any(obj.name_matches_object(name, **kwargs)
                for obj in unique_rng_options.values())
 
 
+def make_name_matcher(*names):
+    """Construct a matcher for custom names
+
+    In case of contexts, there can be custom names. To reuse common cleanup
+    mechanism for cleaning up such resources, this method creates a subclass of
+    RandomNameGeneratorMixin with customized `name_matches_object` method.
+    """
+    class CustomNameMatcher(RandomNameGeneratorMixin):
+        # generate unique string to guarantee processing that custom names
+        RESOURCE_NAME_FORMAT = str(uuid.uuid4())
+
+        NAMES = names
+
+        @classmethod
+        def name_matches_object(cls, name, task_id=None, exact=True):
+            return name in cls.NAMES
+
+    return CustomNameMatcher
+
+
 def merge(length, *sources):
     """Merge lists of lists.
 
@@ -529,17 +585,253 @@ def timeout_thread(queue):
         if not all_threads:
             timeout = None
         else:
-            thread_ident, deadline = all_threads[0]
+            thread, deadline = all_threads[0]
             timeout = deadline - time.time()
         try:
             next_thread = queue.get(timeout=timeout)
             all_threads.append(next_thread)
         except (moves.queue.Empty, ValueError):
             # NOTE(rvasilets) Empty means that timeout was occurred.
-            # ValueError means that timeout lower then 0.
-            LOG.info("Thread %s is timed out. Terminating." % thread_ident)
-            terminate_thread(thread_ident)
+            # ValueError means that timeout lower than 0.
+            if thread.isAlive():
+                LOG.info("Thread %s is timed out. Terminating." % thread.ident)
+                terminate_thread(thread.ident)
             all_threads.popleft()
 
         if next_thread == (None, None,):
             return
+
+
+class LockedDict(dict):
+    """This represents dict which can be locked for updates.
+
+    It is read-only by default, but it can be updated via context manager
+    interface:
+
+    d = LockedDict(foo="bar")
+    d["spam"] = 42  # RuntimeError
+    with d.unlocked():
+         d["spam"] = 42  # Works
+    """
+
+    def __init__(self, *args, **kwargs):
+        super(LockedDict, self).__init__(*args, **kwargs)
+        self._is_locked = True
+        self._is_ready_to_be_unlocked = False
+
+        def lock(obj):
+            if isinstance(obj, dict):
+                return LockedDict(obj)
+            elif isinstance(obj, list):
+                return tuple([lock(v) for v in obj])
+            return obj
+
+        with self.unlocked():
+            for k, v in self.items():
+                self[k] = lock(v)
+
+    def _check_is_unlocked(self):
+        if self._is_locked:
+            raise RuntimeError("Trying to change read-only dict %r" % self)
+
+    def unlocked(self):
+        self._is_ready_to_be_unlocked = True
+        return self
+
+    def __deepcopy__(self, memo=None):
+        def unlock(obj):
+            if isinstance(obj, LockedDict):
+                obj = dict(obj)
+                for k, v in obj.items():
+                    obj[k] = unlock(v)
+            elif type(obj) == tuple:
+                obj = tuple([unlock(v) for v in obj])
+            return obj
+        return copy.deepcopy(unlock(self), memo=memo)
+
+    def __enter__(self, *args):
+        if self._is_ready_to_be_unlocked:
+            self._is_locked = False
+
+    def __exit__(self, *args):
+        self._is_ready_to_be_unlocked = False
+        self._is_locked = True
+
+    def __setitem__(self, *args, **kwargs):
+        self._check_is_unlocked()
+        return super(LockedDict, self).__setitem__(*args, **kwargs)
+
+    def __delitem__(self, *args, **kwargs):
+        self._check_is_unlocked()
+        return super(LockedDict, self).__delitem__(*args, **kwargs)
+
+    def pop(self, *args, **kwargs):
+        self._check_is_unlocked()
+        return super(LockedDict, self).pop(*args, **kwargs)
+
+    def popitem(self, *args, **kwargs):
+        self._check_is_unlocked()
+        return super(LockedDict, self).popitem(*args, **kwargs)
+
+    def update(self, *args, **kwargs):
+        self._check_is_unlocked()
+        return super(LockedDict, self).update(*args, **kwargs)
+
+    def setdefault(self, *args, **kwargs):
+        self._check_is_unlocked()
+        return super(LockedDict, self).setdefault(*args, **kwargs)
+
+    def clear(self, *args, **kwargs):
+        self._check_is_unlocked()
+        return super(LockedDict, self).clear(*args, **kwargs)
+
+
+@logging.log_deprecated(message="Its not used elsewhere in Rally already.",
+                        rally_version="0.11.2")
+def format_float_to_str(num):
+    """DEPRECATED. Use rally.utils.strutils.format_float_to_str instead."""
+    return strutils.format_float_to_str(num)
+
+
+class DequeAsQueue(object):
+    """Allows to use some of Queue methods on collections.deque."""
+
+    def __init__(self, deque):
+        self.deque = deque
+
+    def qsize(self):
+        return len(self.deque)
+
+    def put(self, value):
+        self.deque.append(value)
+
+    def get(self):
+        return self.deque.popleft()
+
+    def empty(self):
+        return bool(self.deque)
+
+
+class Stopwatch(object):
+    """Allows to sleep till specified time since start."""
+
+    def __init__(self, stop_event=None):
+        """Creates Stopwatch.
+
+        :param stop_event: optional threading.Event to use for waiting
+            allows to interrupt sleep. If not provided time.sleep
+            will be used instead.
+        """
+        self._stop_event = stop_event
+
+    def start(self):
+        self._start_time = time.time()
+
+    def sleep(self, sec):
+        """Sleeps till specified second since start."""
+        target_time = self._start_time + sec
+        current_time = time.time()
+        if current_time >= target_time:
+            return
+        time_to_sleep = target_time - current_time
+        self._sleep(time_to_sleep)
+
+    def _sleep(self, sec):
+        if self._stop_event:
+            self._stop_event.wait(sec)
+        else:
+            interruptable_sleep(sec)
+
+
+def generate_random_path(root_dir=None):
+    """Generates a vacant name for a file or dir at the specified place.
+
+    :param root_dir: Name of a directory to generate path in. If None (default
+        behaviour), temporary directory (i.e /tmp in linux) will be used.
+    """
+    root_dir = root_dir or tempfile.gettempdir()
+    path = None
+    while path is None:
+        candidate = os.path.join(root_dir, str(uuid.uuid4()))
+        if not os.path.exists(candidate):
+            path = candidate
+    return path
+
+
+class BackupHelper(object):
+    def __init__(self):
+        self._tempdir = generate_random_path()
+
+        os.mkdir(self._tempdir)
+
+        self._stored_data = {}
+        self._rollback_actions = []
+
+    def backup(self, original_path):
+        if original_path in self._stored_data:
+            raise exceptions.RallyException(
+                "Failed to back up %s since it was already stored."
+                % original_path)
+        backup_path = generate_random_path(self._tempdir)
+        LOG.debug("Creating backup of %s in %s" % (original_path, backup_path))
+        try:
+            shutil.copytree(original_path, backup_path, symlinks=True)
+        except Exception:
+            # Ooops. something went wrong
+            self.rollback()
+            raise
+        self._stored_data[original_path] = backup_path
+
+    def rollback(self):
+        LOG.debug("Performing rollback of stored data.")
+        for original_path, stored_path in self._stored_data.copy().items():
+            if os.path.exists(original_path):
+                shutil.rmtree(original_path)
+            shutil.copytree(stored_path, original_path, symlinks=True)
+            # not to delete the same path in __del__ method
+            self._stored_data.pop(original_path)
+
+        for m, args, kwargs in self._rollback_actions:
+            m(*args, **kwargs)
+
+    def add_rollback_action(self, method, *args, **kwargs):
+        self._rollback_actions.append((method, args, kwargs))
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if exc_type is not None:
+            self.rollback()
+
+    def __call__(self, path):
+        self.backup(path)
+        return self
+
+    def __del__(self):
+        for path in self._stored_data.values():
+            if os.path.exists(path):
+                LOG.debug("Deleting %s" % path)
+                shutil.rmtree(path)
+
+
+def prettify_xml(elem, level=0):
+    """Adds indents.
+
+    Code of this method was copied from
+        http://effbot.org/zone/element-lib.htm#prettyprint
+
+    """
+    i = "\n" + level * "  "
+    if len(elem):
+        if not elem.text or not elem.text.strip():
+            elem.text = i + "  "
+        if not elem.tail or not elem.tail.strip():
+            elem.tail = i
+        for elem in elem:
+            prettify_xml(elem, level + 1)
+        if not elem.tail or not elem.tail.strip():
+            elem.tail = i
+    else:
+        if level and (not elem.tail or not elem.tail.strip()):
+            elem.tail = i
