@@ -25,3 +25,9 @@ version_info = pbr_version.VersionInfo("rally")
 
 def version_string():
     return version_info.semantic_version().debian_string()
+
+
+def database_revision():
+    from rally.common.db import schema
+
+    return schema.schema_revision(detailed=True)
