@@ -14,48 +14,48 @@
 #    under the License.
 
 """
-There is a lot of situation when we would like to work with Enum or consts.
+There is a lot of situations when we would like to work with Enum or Const.
 E.g. work around Tasks. We would like to use Enum in DB to store status of task
 and also in migration that creates DB and in business logic to set some status
-so to avoid copy paste or direct usage of enums values we create singletons
-for each enum. (e.g TaskStatus)
+so as to avoid copy paste or direct usage of enums values we create singletons
+for each enum. (e.g. TaskStatus)
 """
 
 from rally.common import utils
 
 
-class _TempestTestsAPI(utils.ImmutableMixin, utils.EnumMixin):
-    BAREMETAL = "baremetal"
-    COMPUTE = "compute"
-    DATA_PROCESSING = "data_processing"
-    DATABASE = "database"
-    IDENTITY = "identity"
-    IMAGE = "image"
-    MESSAGING = "messaging"
-    NETWORK = "network"
-    OBJECT_STORAGE = "object_storage"
-    ORCHESTRATION = "orchestration"
-    TELEMETRY = "telemetry"
-    VOLUME = "volume"
-
-
-class _TempestTestsSets(utils.ImmutableMixin, utils.EnumMixin):
-    FULL = "full"
-    SMOKE = "smoke"
-    SCENARIO = "scenario"
-
 JSON_SCHEMA = "http://json-schema.org/draft-04/schema"
 
 
 class _TaskStatus(utils.ImmutableMixin, utils.EnumMixin):
+
+    """Consts that represents task possible states."""
     INIT = "init"
-    VERIFYING = "verifying"
-    SETTING_UP = "setting up"
+    VALIDATING = "validating"
+    VALIDATED = "validated"
+    VALIDATION_FAILED = "validation_failed"
     RUNNING = "running"
-    CLEANING_UP = "cleaning up"
     FINISHED = "finished"
-    FAILED = "failed"
+    CRASHED = "crashed"
     ABORTING = "aborting"
+    SLA_FAILED = "sla_failed"
+    SOFT_ABORTING = "soft_aborting"
+    ABORTED = "aborted"
+    PAUSED = "paused"
+
+
+class _SubtaskStatus(utils.ImmutableMixin, utils.EnumMixin):
+
+    """Consts that represents task possible states."""
+    INIT = "init"
+    VALIDATING = "validating"
+    VALIDATED = "validated"
+    VALIDATION_FAILED = "validation_failed"
+    RUNNING = "running"
+    FINISHED = "finished"
+    CRASHED = "crashed"
+    ABORTING = "aborting"
+    SLA_FAILED = "sla_failed"
     SOFT_ABORTING = "soft_aborting"
     ABORTED = "aborted"
     PAUSED = "paused"
@@ -103,12 +103,16 @@ class _Service(utils.ImmutableMixin, utils.EnumMixin):
     CEILOMETER = "ceilometer"
     MONASCA = "monasca"
     S3 = "s3"
+    SENLIN = "senlin"
     TROVE = "trove"
     SAHARA = "sahara"
     SWIFT = "swift"
     MISTRAL = "mistral"
     MURANO = "murano"
     IRONIC = "ironic"
+    GNOCCHI = "gnocchi"
+    MAGNUM = "magnum"
+    WATCHER = "watcher"
 
 
 class _ServiceType(utils.ImmutableMixin, utils.EnumMixin):
@@ -121,6 +125,7 @@ class _ServiceType(utils.ImmutableMixin, utils.EnumMixin):
     CLOUD = "cloudformation"
     ORCHESTRATION = "orchestration"
     IDENTITY = "identity"
+    CLUSTERING = "clustering"
     COMPUTE = "compute"
     NETWORK = "network"
     DNS = "dns"
@@ -134,9 +139,13 @@ class _ServiceType(utils.ImmutableMixin, utils.EnumMixin):
     WORKFLOW_EXECUTION = "workflowv2"
     APPLICATION_CATALOG = "application-catalog"
     BARE_METAL = "baremetal"
+    METRIC = "metric"
+    CONTAINER_INFRA = "container-infra"
+    INFRA_OPTIM = "infra-optim"
 
     def __init__(self):
         self.__names = {
+            self.CLUSTERING: _Service.SENLIN,
             self.COMPUTE: _Service.NOVA,
             self.VOLUME: _Service.CINDER,
             self.SHARE: _Service.MANILA,
@@ -157,6 +166,9 @@ class _ServiceType(utils.ImmutableMixin, utils.EnumMixin):
             self.WORKFLOW_EXECUTION: _Service.MISTRAL,
             self.APPLICATION_CATALOG: _Service.MURANO,
             self.BARE_METAL: _Service.IRONIC,
+            self.METRIC: _Service.GNOCCHI,
+            self.CONTAINER_INFRA: _Service.MAGNUM,
+            self.INFRA_OPTIM: _Service.WATCHER,
         }
 
     def __getitem__(self, service_type):
@@ -168,11 +180,54 @@ class _ServiceType(utils.ImmutableMixin, utils.EnumMixin):
         return self.__names[service_type]
 
 
+class _HookStatus(utils.ImmutableMixin, utils.EnumMixin):
+    """Hook result statuses."""
+    SUCCESS = "success"
+    FAILED = "failed"
+    VALIDATION_FAILED = "validation_failed"
+
+
+class _TagType(utils.ImmutableMixin, utils.EnumMixin):
+    TASK = "task"
+    SUBTASK = "subtask"
+    VERIFICATION = "verification"
+
+
+class _VerifierStatus(utils.ImmutableMixin, utils.EnumMixin):
+    """Verifier statuses."""
+    INIT = "init"
+    INSTALLING = "installing"
+    INSTALLED = "installed"
+    UPDATING = "updating"
+    EXTENDING = "extending"
+    FAILED = "failed"
+
+
+# NOTE(andreykurilin): In case of updating these statuses, please do not forget
+#   to update doc reference too
+class _VerificationStatus(utils.ImmutableMixin, utils.EnumMixin):
+    """Verification statuses."""
+    INIT = "init"
+    RUNNING = "running"
+    FINISHED = "finished"
+    FAILED = "failed"
+    CRASHED = "crashed"
+
+
+class _TimeFormat(utils.ImmutableMixin, utils.EnumMixin):
+    """International time formats"""
+    ISO8601 = "%Y-%m-%dT%H:%M:%S%z"
+
+
 TaskStatus = _TaskStatus()
+SubtaskStatus = _SubtaskStatus()
 DeployStatus = _DeployStatus()
 EndpointPermission = _EndpointPermission()
 ServiceType = _ServiceType()
 Service = _Service()
 EndpointType = _EndpointType()
-TempestTestsAPI = _TempestTestsAPI()
-TempestTestsSets = _TempestTestsSets()
+HookStatus = _HookStatus()
+TagType = _TagType()
+VerifierStatus = _VerifierStatus()
+VerificationStatus = _VerificationStatus()
+TimeFormat = _TimeFormat()
