@@ -20,17 +20,16 @@ import time
 
 from six.moves import queue as Queue
 
-from rally.common import logging
 from rally.common import utils
+from rally.common import validation
 from rally import consts
 from rally.task import runner
 from rally.task import utils as butils
 
-LOG = logging.getLogger(__name__)
-
 
 def _worker_process(queue, iteration_gen, timeout, concurrency, times,
-                    context, cls, method_name, args, aborted, info):
+                    context, cls, method_name, args, event_queue, aborted,
+                    info):
     """Start the scenario within threads.
 
     Spawn threads to support scenario execution for a fixed number of times.
@@ -48,6 +47,7 @@ def _worker_process(queue, iteration_gen, timeout, concurrency, times,
     :param cls: scenario class
     :param method_name: scenario method name
     :param args: scenario args
+    :param event_queue: queue object to append events
     :param aborted: multiprocessing.Event that aborts load generation if
                     the flag is set
     :param info: info about all processes count and counter of launched process
@@ -71,24 +71,24 @@ def _worker_process(queue, iteration_gen, timeout, concurrency, times,
 
     iteration = next(iteration_gen)
     while iteration < times and not aborted.is_set():
-        scenario_context = runner._get_scenario_context(context)
-        scenario_args = (iteration, cls, method_name, scenario_context, args)
-        worker_args = (queue, scenario_args)
+        scenario_context = runner._get_scenario_context(iteration, context)
+        worker_args = (
+            queue, cls, method_name, scenario_context, args, event_queue)
 
         thread = threading.Thread(target=runner._worker_thread,
                                   args=worker_args)
 
         thread.start()
         if timeout:
-            timeout_queue.put((thread.ident, time.time() + timeout))
-        pool.append((thread, time.time()))
+            timeout_queue.put((thread, time.time() + timeout))
+        pool.append(thread)
         alive_threads_in_pool += 1
 
         while alive_threads_in_pool == concurrency:
             prev_finished_threads_in_pool = finished_threads_in_pool
             finished_threads_in_pool = 0
             for t in pool:
-                if not t[0].isAlive():
+                if not t.isAlive():
                     finished_threads_in_pool += 1
 
             alive_threads_in_pool -= finished_threads_in_pool
@@ -98,8 +98,8 @@ def _worker_process(queue, iteration_gen, timeout, concurrency, times,
                 # NOTE(boris-42): cleanup pool array. This is required because
                 # in other case array length will be equal to times which
                 # is unlimited big
-                while pool and not pool[0][0].isAlive():
-                    pool.popleft()[0].join()
+                while pool and not pool[0].isAlive():
+                    pool.popleft().join()
                     finished_threads_in_pool -= 1
                 break
 
@@ -109,13 +109,28 @@ def _worker_process(queue, iteration_gen, timeout, concurrency, times,
 
     # Wait until all threads are done
     while pool:
-        pool.popleft()[0].join()
+        pool.popleft().join()
 
     if timeout:
         timeout_queue.put((None, None,))
         collector_thr_by_timeout.join()
 
 
+@validation.configure("check_constant")
+class CheckConstantValidator(validation.Validator):
+    """Additional schema validation for constant runner"""
+
+    def validate(self, context, config, plugin_cls, plugin_cfg):
+        if plugin_cfg.get("concurrency", 1) > plugin_cfg.get("times", 1):
+            return self.fail(
+                "Parameter 'concurrency' means a number of parallel executions"
+                "of iterations. Parameter 'times' means total number of "
+                "iteration executions. It is redundant (and restricted) to "
+                "have number of parallel iterations bigger then total number "
+                "of iterations.")
+
+
+@validation.add("check_constant")
 @runner.configure(name="constant")
 class ConstantScenarioRunner(runner.ScenarioRunner):
     """Creates constant load executing a scenario a specified number of times.
@@ -125,8 +140,8 @@ class ConstantScenarioRunner(runner.ScenarioRunner):
     up to the number of times specified in the scenario config.
 
     The concurrency parameter of the scenario config controls the
-    number of concurrent scenarios which execute during a single
-    iteration in order to simulate the activities of multiple users
+    number of concurrent iterations which execute during a single
+    scenario in order to simulate the activities of multiple users
     placing load on the cloud under test.
     """
 
@@ -134,32 +149,32 @@ class ConstantScenarioRunner(runner.ScenarioRunner):
         "type": "object",
         "$schema": consts.JSON_SCHEMA,
         "properties": {
-            "type": {
-                "type": "string"
-            },
             "concurrency": {
                 "type": "integer",
-                "minimum": 1
+                "minimum": 1,
+                "description": "The number of parallel iteration executions."
             },
             "times": {
                 "type": "integer",
-                "minimum": 1
+                "minimum": 1,
+                "description": "Total number of iteration executions."
             },
             "timeout": {
                 "type": "number",
-                "minimum": 1
+                "description": "Operation's timeout."
             },
             "max_cpu_count": {
                 "type": "integer",
-                "minimum": 1
+                "minimum": 1,
+                "description": "The maximum number of processes to create load"
+                               " from."
             }
         },
-        "required": ["type"],
         "additionalProperties": False
     }
 
     def _run_scenario(self, cls, method_name, context, args):
-        """Runs the specified benchmark scenario with given arguments.
+        """Runs the specified scenario with given arguments.
 
         This method generates a constant load on the cloud under test by
         executing each scenario iteration using a pool of processes without
@@ -168,8 +183,9 @@ class ConstantScenarioRunner(runner.ScenarioRunner):
 
         :param cls: The Scenario class where the scenario is implemented
         :param method_name: Name of the method that implements the scenario
-        :param context: Benchmark context that contains users, admin & other
-                        information, that was created before benchmark started.
+        :param context: context that contains users, admin & other
+                        information, that was created before scenario
+                        execution starts.
         :param args: Arguments to call the scenario method with
 
         :returns: List of results fore each single scenario iteration,
@@ -195,19 +211,33 @@ class ConstantScenarioRunner(runner.ScenarioRunner):
                              concurrency_overhead=concurrency_overhead)
 
         result_queue = multiprocessing.Queue()
+        event_queue = multiprocessing.Queue()
 
         def worker_args_gen(concurrency_overhead):
             while True:
                 yield (result_queue, iteration_gen, timeout,
                        concurrency_per_worker + (concurrency_overhead and 1),
-                       times, context, cls, method_name, args, self.aborted)
+                       times, context, cls, method_name, args, event_queue,
+                       self.aborted)
                 if concurrency_overhead:
                     concurrency_overhead -= 1
 
         process_pool = self._create_process_pool(
             processes_to_start, _worker_process,
             worker_args_gen(concurrency_overhead))
-        self._join_processes(process_pool, result_queue)
+        self._join_processes(process_pool, result_queue, event_queue)
+
+
+def _run_scenario_once_with_unpack_args(args):
+    # NOTE(andreykurilin): `pool.imap` is used in
+    #     ConstantForDurationScenarioRunner. It does not want to work with
+    #     instance-methods, class-methods and static-methods. Also, it can't
+    #     transmit positional or keyword arguments to destination function.
+    #     While original `rally.task.runner._run_scenario_once` accepts
+    #     multiple arguments instead of one big tuple with all arguments, we
+    #     need to hardcode unpacking here(all other runners are able to
+    #     transmit arguments in proper way).
+    return runner._run_scenario_once(*args)
 
 
 @runner.configure(name="constant_for_duration")
@@ -219,8 +249,8 @@ class ConstantForDurationScenarioRunner(runner.ScenarioRunner):
     until a specified interval of time has elapsed.
 
     The concurrency parameter of the scenario config controls the
-    number of concurrent scenarios which execute during a single
-    iteration in order to simulate the activities of multiple users
+    number of concurrent iterations which execute during a single
+    sceanario in order to simulate the activities of multiple users
     placing load on the cloud under test.
     """
 
@@ -228,41 +258,44 @@ class ConstantForDurationScenarioRunner(runner.ScenarioRunner):
         "type": "object",
         "$schema": consts.JSON_SCHEMA,
         "properties": {
-            "type": {
-                "type": "string"
-            },
             "concurrency": {
                 "type": "integer",
-                "minimum": 1
+                "minimum": 1,
+                "description": "The number of parallel iteration executions."
             },
             "duration": {
                 "type": "number",
-                "minimum": 0.0
+                "minimum": 0.0,
+                "description": "The number of seconds during which to generate"
+                               " a load."
             },
             "timeout": {
                 "type": "number",
-                "minimum": 1
+                "minimum": 1,
+                "description": "Operation's timeout."
             }
         },
-        "required": ["type", "duration"],
+        "required": ["duration"],
         "additionalProperties": False
     }
 
     @staticmethod
-    def _iter_scenario_args(cls, method, ctx, args, aborted):
+    def _iter_scenario_args(cls, method, ctx, args, event_queue, aborted):
         def _scenario_args(i):
             if aborted.is_set():
                 raise StopIteration()
-            return (i, cls, method, runner._get_scenario_context(ctx), args)
+            return (cls, method, runner._get_scenario_context(i, ctx), args,
+                    event_queue)
         return _scenario_args
 
     def _run_scenario(self, cls, method, context, args):
-        """Runs the specified benchmark scenario with given arguments.
+        """Runs the specified scenario with given arguments.
 
         :param cls: The Scenario class where the scenario is implemented
-        :param method_name: Name of the method that implements the scenario
-        :param context: Benchmark context that contains users, admin & other
-                        information, that was created before benchmark started.
+        :param method: Name of the method that implements the scenario
+        :param context: context that contains users, admin & other
+                        information, that was created before scenario is
+                        execution starts.
         :param args: Arguments to call the scenario method with
 
         :returns: List of results fore each single scenario iteration,
@@ -272,27 +305,46 @@ class ConstantForDurationScenarioRunner(runner.ScenarioRunner):
         concurrency = self.config.get("concurrency", 1)
         duration = self.config.get("duration")
 
+        # FIXME(andreykurilin): unify `_worker_process`, use it here and remove
+        #     usage of `multiprocessing.Pool`(usage of separate process for
+        #     each concurrent iteration is redundant).
         pool = multiprocessing.Pool(concurrency)
+        manager = multiprocessing.Manager()
+        event_queue = manager.Queue()
+        stop_event_listener = threading.Event()
+
+        def event_listener():
+            while not stop_event_listener.isSet():
+                while not event_queue.empty():
+                    self.send_event(**event_queue.get())
+                else:
+                    time.sleep(0.01)
+
+        event_listener_thread = threading.Thread(target=event_listener)
+        event_listener_thread.start()
 
         run_args = butils.infinite_run_args_generator(
-            self._iter_scenario_args(cls, method, context, args,
-                                     self.aborted))
-        iter_result = pool.imap(runner._run_scenario_once, run_args)
+            self._iter_scenario_args(
+                cls, method, context, args, event_queue, self.aborted))
+        iter_result = pool.imap(_run_scenario_once_with_unpack_args, run_args)
 
         start = time.time()
-        while True:
-            try:
-                result = iter_result.next(timeout)
-            except multiprocessing.TimeoutError as e:
-                result = runner.format_result_on_timeout(e, timeout)
-            except StopIteration:
-                break
+        try:
+            while True:
+                try:
+                    result = iter_result.next(timeout)
+                except multiprocessing.TimeoutError as e:
+                    result = runner.format_result_on_timeout(e, timeout)
+                except StopIteration:
+                    break
 
-            self._send_result(result)
+                self._send_result(result)
 
-            if time.time() - start > duration:
-                break
-
-        pool.terminate()
-        pool.join()
-        self._flush_results()
+                if time.time() - start > duration:
+                    break
+        finally:
+            stop_event_listener.set()
+            event_listener_thread.join()
+            pool.terminate()
+            pool.join()
+            self._flush_results()
