@@ -19,7 +19,6 @@ SLA (Service-level agreement) is set of details for determining compliance
 with contracted values such as maximum error rate or minimum response time.
 """
 
-from rally.common.i18n import _
 from rally.common import streaming_algorithms
 from rally.task import sla
 
@@ -49,5 +48,5 @@ class MaxAverageDuration(sla.SLA):
         return self.success
 
     def details(self):
-        return (_("Average duration of one iteration %.2fs <= %.2fs - %s") %
+        return ("Average duration of one iteration %.2fs <= %.2fs - %s" %
                 (self.avg, self.criterion_value, self.status()))
