@@ -13,21 +13,28 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
 import functools
 
-from rally.common import costilius
+from rally.common import logging
 from rally.common import utils
 
+LOG = logging.getLogger(__name__)
+
 
 class ActionTimerMixin(object):
 
     def __init__(self):
-        self._atomic_actions = costilius.OrderedDict()
+        self._atomic_actions = []
 
     def atomic_actions(self):
         """Returns the content of each atomic action."""
         return self._atomic_actions
 
+    def reset_atomic_actions(self):
+        """Clean all atomic action data."""
+        self._atomic_actions = []
+
 
 class ActionTimer(utils.Timer):
     """A class to measure the duration of atomic operations
@@ -48,27 +55,27 @@ class ActionTimer(utils.Timer):
         """
         super(ActionTimer, self).__init__()
         self.instance = instance
-        self.name = self._get_atomic_action_name(instance, name)
-        self.instance._atomic_actions[self.name] = None
+        self.name = name
+        self._root = self._find_parent(self.instance._atomic_actions)
+        self.atomic_action = {"name": self.name,
+                              "children": [],
+                              "started_at": None}
+        self._root.append(self.atomic_action)
 
-    @classmethod
-    def _get_atomic_action_name(cls, instance, name):
-        # TODO(boris-42): It was quite bad idea to store atomic actions
-        #                 inside {}. We should refactor this in 0.2.0 release
-        #                 and store them inside array, that will allow us to
-        #                 store atomic actions with the same name
-        if name not in instance._atomic_actions:
-            return name
+    def _find_parent(self, atomic_actions):
+        while atomic_actions and "finished_at" not in atomic_actions[-1]:
+            atomic_actions = atomic_actions[-1]["children"]
+        return atomic_actions
 
-        name_template = name + " (%i)"
-        i = 2
-        while name_template % i in instance._atomic_actions:
-            i += 1
-        return name_template % i
+    def __enter__(self):
+        super(ActionTimer, self).__enter__()
+        self.atomic_action["started_at"] = self.start
 
     def __exit__(self, type_, value, tb):
         super(ActionTimer, self).__exit__(type_, value, tb)
-        self.instance._atomic_actions[self.name] = self.duration()
+        self.atomic_action["finished_at"] = self.finish
+        if type_:
+            self.atomic_action["failed"] = True
 
 
 def action_timer(name):
@@ -108,6 +115,12 @@ def optional_action_timer(name, argument_name="atomic_action", default=True):
     def wrap(func):
         @functools.wraps(func)
         def func_atomic_actions(self, *args, **kwargs):
+            LOG.warning("'optional_action_timer' is deprecated"
+                        "since rally v0.10.0."
+                        "Please use action_timer instead, "
+                        "we have improved atomic actions, "
+                        "now do not need to explicitly close "
+                        "original action.")
             if kwargs.pop(argument_name, default):
                 with ActionTimer(self, name):
                     f = func(self, *args, **kwargs)
@@ -116,3 +129,36 @@ def optional_action_timer(name, argument_name="atomic_action", default=True):
             return f
         return func_atomic_actions
     return wrap
+
+
+def merge_atomic_actions(atomic_actions, root=None, depth=0,
+                         depth_of_processing=2):
+    """Merge duplicates of atomic actions into one atomic action.
+
+    :param atomic_actions: a list with atomic action
+    :param root: an ordered dict to save atomics to (leave it with a default
+        value, since the primary use case of that parameter is processing inner
+        atomic actions)
+    :param depth: current level of processing inner atomic actions
+    :param depth_of_processing: the depth of processing of inner atomic actions
+        (defaults to 2)
+    """
+    p_atomics = collections.OrderedDict() if root is None else root
+    for action in atomic_actions:
+        if action["name"] not in p_atomics:
+            p_atomics[action["name"]] = {
+                "duration": 0,
+                "count": 0,
+                "children": collections.OrderedDict()}
+        duration = action["finished_at"] - action["started_at"]
+        p_atomics[action["name"]]["duration"] += duration
+        p_atomics[action["name"]]["count"] += 1
+        if action.get("failed"):
+            p_atomics[action["name"]]["failed"] = True
+        if action["children"] and depth < depth_of_processing:
+            merge_atomic_actions(
+                action["children"],
+                root=p_atomics[action["name"]]["children"],
+                depth=depth + 1)
+
+    return p_atomics
