@@ -15,23 +15,23 @@
 
 import abc
 import collections
+import copy
 import multiprocessing
 import time
 
-import jsonschema
+import six
 
 from rally.common import logging
-from rally.common import objects
 from rally.common.plugin import plugin
 from rally.common import utils as rutils
-from rally import consts
-from rally.task import context
+from rally.common import validation
 from rally.task import scenario
 from rally.task import types
 from rally.task import utils
 
 
 LOG = logging.getLogger(__name__)
+configure = plugin.configure
 
 
 def format_result_on_timeout(exc, timeout):
@@ -39,53 +39,40 @@ def format_result_on_timeout(exc, timeout):
         "duration": timeout,
         "idle_duration": 0,
         "output": {"additive": [], "complete": []},
-        "atomic_actions": {},
+        "atomic_actions": [],
         "error": utils.format_exc(exc)
     }
 
 
-def _get_scenario_context(context_obj):
-    return context.ContextManager(context_obj).map_for_scenario()
+def _get_scenario_context(iteration, context_obj):
+    context_obj = copy.deepcopy(context_obj)
+    context_obj["iteration"] = iteration + 1  # Numeration starts from `1'
+    return context_obj
 
 
-def _run_scenario_once(args):
-    iteration, cls, method_name, context_obj, kwargs = args
+def _run_scenario_once(cls, method_name, context_obj, scenario_kwargs,
+                       event_queue):
+    iteration = context_obj["iteration"]
+    event_queue.put({
+        "type": "iteration",
+        "value": iteration,
+    })
+
+    # provide arguments isolation between iterations
+    scenario_kwargs = copy.deepcopy(scenario_kwargs)
 
     LOG.info("Task %(task)s | ITER: %(iteration)s START" %
              {"task": context_obj["task"]["uuid"], "iteration": iteration})
 
-    context_obj["iteration"] = iteration
     scenario_inst = cls(context_obj)
-
     error = []
-    output = {"additive": [], "complete": []}
     try:
         with rutils.Timer() as timer:
-            # NOTE(amaretskiy): Output as return value is deprecated
-            #     but supported for backward compatibility
-            deprecated_output = getattr(scenario_inst, method_name)(**kwargs)
-            warning = ""
-            if deprecated_output:
-                warning = ("Returning output data by scenario is deprecated "
-                           "in favor of calling add_output().")
-            if scenario_inst._output != {"complete": [], "additive": []}:
-                output = scenario_inst._output
-                if deprecated_output:
-                    warning += (" Output data both returned and passed to "
-                                "add_output() so returned one is ignored!")
-            elif deprecated_output:
-                output["additive"].append({
-                    "title": "Scenario output",
-                    "description": "",
-                    "chart_plugin": "StackedArea",
-                    "data": [list(item)
-                             for item in deprecated_output["data"].items()]})
-            if warning:
-                LOG.warning(warning)
+            getattr(scenario_inst, method_name)(**scenario_kwargs)
     except Exception as e:
         error = utils.format_exc(e)
         if logging.is_debug():
-            LOG.exception(e)
+            LOG.exception("Iteration %s raised Exception" % iteration)
     finally:
         status = "Error %s: %s" % tuple(error[0:2]) if error else "OK"
         LOG.info("Task %(task)s | ITER: %(iteration)s END: %(status)s" %
@@ -96,12 +83,14 @@ def _run_scenario_once(args):
                 "timestamp": timer.timestamp(),
                 "idle_duration": scenario_inst.idle_duration(),
                 "error": error,
-                "output": output,
+                "output": scenario_inst._output,
                 "atomic_actions": scenario_inst.atomic_actions()}
 
 
-def _worker_thread(queue, args):
-    queue.put(_run_scenario_once(args))
+def _worker_thread(queue, cls, method_name, context_obj, scenario_kwargs,
+                   event_queue):
+    queue.put(_run_scenario_once(cls, method_name, context_obj,
+                                 scenario_kwargs, event_queue))
 
 
 def _log_worker_info(**info):
@@ -114,60 +103,27 @@ def _log_worker_info(**info):
     LOG.debug("Starting a worker.\n\t%s" % info_message)
 
 
-class ScenarioRunnerResult(dict):
-    """Class for all scenario runners' result."""
-
-    RESULT_SCHEMA = {
-        "type": "object",
-        "$schema": consts.JSON_SCHEMA,
-        "properties": {
-            "duration": {
-                "type": "number"
-            },
-            "timestamp": {
-                "type": "number"
-            },
-            "idle_duration": {
-                "type": "number"
-            },
-            "output": objects.task.OUTPUT_SCHEMA,
-            "atomic_actions": {
-                "type": "object",
-                "patternProperties": {
-                    ".*": {"type": ["number", "null"]}
-                }
-            },
-            "error": {
-                "type": "array",
-                "items": {
-                    "type": "string"
-                }
-            }
-        },
-        "additionalProperties": False
-    }
-
-    def __init__(self, result_list):
-        super(ScenarioRunnerResult, self).__init__(result_list)
-        jsonschema.validate(result_list, self.RESULT_SCHEMA)
-
-
-def configure(name, namespace="default"):
-    return plugin.configure(name=name, namespace=namespace)
-
-
-@configure(name="base_runner")
-class ScenarioRunner(plugin.Plugin):
+@validation.add_default("jsonschema")
+@plugin.base()
+@six.add_metaclass(abc.ABCMeta)
+class ScenarioRunner(plugin.Plugin, validation.ValidatablePluginMixin):
     """Base class for all scenario runners.
 
     Scenario runner is an entity that implements a certain strategy of
-    launching benchmark scenarios, e.g. running them continuously or
+    launching scenarios plugins, e.g. running them continuously or
     periodically for a given number of times or seconds.
     These strategies should be implemented in subclasses of ScenarioRunner
     in the_run_scenario() method.
     """
 
-    CONFIG_SCHEMA = {}
+    CONFIG_SCHEMA = {
+        "type": "object",
+        "properties": {
+            "type": {"type": "string"},
+        },
+        "required": ["type"],
+        "additionalProperties": True
+    }
 
     def __init__(self, task, config, batch_size=0):
         """Runner constructor.
@@ -176,51 +132,45 @@ class ScenarioRunner(plugin.Plugin):
         result_queue, where results will be put by _send_result method.
 
         :param task: Instance of objects.Task
-        :param config: Dict with runner section from benchmark configuration
+        :param config: Dict with runner section of input task
         """
         self.task = task
         self.config = config
         self.result_queue = collections.deque()
+        self.event_queue = collections.deque()
         self.aborted = multiprocessing.Event()
         self.run_duration = 0
         self.batch_size = batch_size
         self.result_batch = []
 
-    @staticmethod
-    def validate(config):
-        """Validates runner's part of task config."""
-        runner = ScenarioRunner.get(config.get("type", "serial"))
-        jsonschema.validate(config, runner.CONFIG_SCHEMA)
-
     @abc.abstractmethod
     def _run_scenario(self, cls, method_name, context, args):
-        """Runs the specified benchmark scenario with given arguments.
+        """Runs the specified scenario with given arguments.
 
         :param cls: The Scenario class where the scenario is implemented
         :param method_name: Name of the method that implements the scenario
-        :param context: Benchmark context that contains users, admin & other
-                        information, that was created before benchmark started.
-        :param args: Arguments to call the scenario method with
+        :param context: dict object that contains data created
+                        by contexts plugins
+        :param args: Arguments to call the scenario plugin with
 
         :returns: List of results fore each single scenario iteration,
                   where each result is a dictionary
         """
 
     def run(self, name, context, args):
-        cls_name, method_name = name.split(".", 1)
-        cls = scenario.Scenario.get(name)._meta_get("cls_ref")
+        scenario_plugin = scenario.Scenario.get(name)
 
         # NOTE(boris-42): processing @types decorators
         args = types.preprocess(name, context, args)
 
         with rutils.Timer() as timer:
-            self._run_scenario(cls, method_name, context, args)
+            # TODO(boris-42): remove method_name argument, now it's always run
+            self._run_scenario(scenario_plugin, "run", context, args)
 
         self.run_duration = timer.duration()
-        return self.run_duration
 
     def abort(self):
-        """Abort the execution of further benchmark scenario iterations."""
+        """Abort the execution of further scenario iterations."""
         self.aborted.set()
 
     @staticmethod
@@ -246,31 +196,36 @@ class ScenarioRunner(plugin.Plugin):
 
         return process_pool
 
-    def _join_processes(self, process_pool, result_queue):
+    def _join_processes(self, process_pool, result_queue, event_queue):
         """Join the processes in the pool and send their results to the queue.
 
         :param process_pool: pool of processes to join
-        :result_queue: multiprocessing.Queue that receives the results
+        :param result_queue: multiprocessing.Queue that receives the results
+        :param event_queue: multiprocessing.Queue that receives the events
         """
         while process_pool:
             while process_pool and not process_pool[0].is_alive():
                 process_pool.popleft().join()
 
-            if result_queue.empty():
+            if result_queue.empty() and event_queue.empty():
                 # sleep a bit to avoid 100% usage of CPU by this method
-                time.sleep(0.001)
+                time.sleep(0.01)
+
+            while not event_queue.empty():
+                self.send_event(**event_queue.get())
 
             while not result_queue.empty():
                 self._send_result(result_queue.get())
 
         self._flush_results()
         result_queue.close()
+        event_queue.close()
 
     def _flush_results(self):
         if self.result_batch:
             sorted_batch = sorted(self.result_batch)
             self.result_queue.append(sorted_batch)
-            self.result_batch = []
+            del self.result_batch[:]
 
     def _send_result(self, result):
         """Store partial result to send it to consumer later.
@@ -280,14 +235,29 @@ class ScenarioRunner(plugin.Plugin):
                        ValidationError is raised.
         """
 
-        r = ScenarioRunnerResult(result)
-        self.result_batch.append(r)
+        if not self.task.result_has_valid_schema(result):
+            LOG.warning(
+                "Task %(task)s | Runner `%(runner)s` is trying to send "
+                "results in wrong format"
+                % {"task": self.task["uuid"], "runner": self.get_name()})
+            return
+
+        self.result_batch.append(result)
 
         if len(self.result_batch) >= self.batch_size:
             sorted_batch = sorted(self.result_batch,
-                                  key=lambda r: r["timestamp"])
+                                  key=lambda r: result["timestamp"])
             self.result_queue.append(sorted_batch)
-            self.result_batch = []
+            del self.result_batch[:]
+
+    def send_event(self, type, value=None):
+        """Store event to send it to consumer later.
+
+        :param type: Event type
+        :param value: Optional event data
+        """
+        self.event_queue.append({"type": type,
+                                 "value": value})
 
     def _log_debug_info(self, **info):
         """Log runner parameters for debugging.
@@ -300,7 +270,7 @@ class ScenarioRunner(plugin.Plugin):
         info_message = "\n\t".join(["%s: %s" % (k, v)
                                     for k, v in info.items()])
         LOG.debug("Starting the %(name)s runner (task UUID: %(task)s)."
-                  "\n\t%(info)s" %
-                  {"name": self._meta_get("name"),
-                   "task": self.task["uuid"],
-                   "info": info_message})
+                  "\n\t%(info)s"
+                  % {"name": self._meta_get("name"),
+                     "task": self.task["uuid"],
+                     "info": info_message})
