@@ -21,11 +21,13 @@ with contracted values such as maximum error rate or minimum response time.
 
 import abc
 
-import jsonschema
 import six
 
-from rally.common.i18n import _
 from rally.common.plugin import plugin
+from rally.common import validation
+
+
+configure = plugin.configure
 
 
 def _format_result(criterion_name, success, detail):
@@ -75,28 +77,27 @@ class SLAChecker(object):
         self_config = self.config.get("sla", {})
         other_config = other.config.get("sla", {})
         if self_config != other_config:
-            message = _(
+            raise TypeError(
                 "Error merging SLACheckers with configs %s, %s. "
                 "Only SLACheckers with the same config could be merged."
-            ) % (self_config, other_config)
-            raise TypeError(message)
+                % (self_config, other_config))
 
     def results(self):
         results = [sla.result() for sla in self.sla_criteria]
         if self.aborted_on_sla:
             results.append(_format_result(
                 "aborted_on_sla", False,
-                _("Task was aborted due to SLA failure(s).")))
+                "Task was aborted due to SLA failure(s)."))
 
         if self.aborted_manually:
             results.append(_format_result(
                 "aborted_manually", False,
-                _("Task was aborted due to abort signal.")))
+                "Task was aborted due to abort signal."))
 
         if self.unexpected_failure:
             results.append(_format_result(
                 "something_went_wrong", False,
-                _("Unexpected error: %s") % self.unexpected_failure))
+                "Unexpected error: %s" % self.unexpected_failure))
 
         return results
 
@@ -110,30 +111,18 @@ class SLAChecker(object):
         self.unexpected_failure = exc
 
 
-def configure(name, namespace="default"):
-    return plugin.configure(name=name, namespace=namespace)
-
-
+@validation.add_default("jsonschema")
+@plugin.base()
 @six.add_metaclass(abc.ABCMeta)
-@configure(name="base_sla")
-class SLA(plugin.Plugin):
+class SLA(plugin.Plugin, validation.ValidatablePluginMixin):
     """Factory for criteria classes."""
 
+    CONFIG_SCHEMA = {"type": "null"}
+
     def __init__(self, criterion_value):
         self.criterion_value = criterion_value
         self.success = True
 
-    @staticmethod
-    def validate(config):
-        properties = dict([(s.get_name(), s.CONFIG_SCHEMA)
-                           for s in SLA.get_all()])
-        schema = {
-            "type": "object",
-            "properties": properties,
-            "additionalProperties": False,
-        }
-        jsonschema.validate(config, schema)
-
     @abc.abstractmethod
     def add_iteration(self, iteration):
         """Process the result of a single iteration and perform a SLA check.
@@ -192,8 +181,6 @@ class SLA(plugin.Plugin):
 
     def validate_type(self, other):
         if type(self) != type(other):
-            message = _(
-                "Error merging SLAs of types %s, %s. "
-                "Only SLAs of the same type could be merged."
-            ) % (type(self), type(other))
-            raise TypeError(message)
+            raise TypeError(
+                "Error merging SLAs of types %s, %s. Only SLAs of the same "
+                "type could be merged." % (type(self), type(other)))
