@@ -28,6 +28,11 @@ import functools
 import re
 import tokenize
 
+
+re_assert_equal_end_with_true_or_false = re.compile(
+    r"assertEqual\(.*?, \s+(True|False)\)$")
+re_assert_equal_start_with_true_or_false = re.compile(
+    r"assertEqual\((True|False),")
 re_assert_true_instance = re.compile(
     r"(.)*assertTrue\(isinstance\((\w|\.|\'|\"|\[|\])+, "
     r"(\w|\.|\'|\"|\[|\])+\)\)")
@@ -36,6 +41,9 @@ re_assert_equal_type = re.compile(
     r"(\w|\.|\'|\"|\[|\])+\)")
 re_assert_equal_end_with_none = re.compile(r"assertEqual\(.*?,\s+None\)$")
 re_assert_equal_start_with_none = re.compile(r"assertEqual\(None,")
+re_assert_not_equal_end_with_none = re.compile(
+    r"assertNotEqual\(.*?,\s+None\)$")
+re_assert_not_equal_start_with_none = re.compile(r"assertNotEqual\(None,")
 re_assert_true_false_with_in_or_not_in = re.compile(
     r"assert(True|False)\("
     r"(\w|[][.'\"])+( not)? in (\w|[][.'\",])+(, .*)?\)")
@@ -63,6 +71,9 @@ re_raises = re.compile(
     r"\s:raise[^s] *.*$|\s:raises *:.*$|\s:raises *[^:]+$")
 re_db_import = re.compile(r"^from rally.common import db")
 re_objects_import = re.compile(r"^from rally.common import objects")
+re_old_type_class = re.compile(r"^\s*class \w+(\(\))?:")
+re_datetime_alias = re.compile(r"^(from|import) datetime(?!\s+as\s+dt$)")
+re_log_warn = re.compile(r"(.)*LOG\.(warn)\(\s*('|\"|_)")
 
 
 def skip_ignored_lines(func):
@@ -80,6 +91,9 @@ def skip_ignored_lines(func):
 def _parse_assert_mock_str(line):
     point = line.find(".assert_")
 
+    if point == -1:
+        point = line.find(".called_once_with(")
+
     if point != -1:
         end_pos = line[point:].find("(") + point
         return point, line[point + 1: end_pos], line[: point]
@@ -94,10 +108,12 @@ def check_assert_methods_from_mock(logical_line, physical_line, filename):
     N301 - base error number
     N302 - related to nonexistent "assert_called"
     N303 - related to nonexistent "assert_called_once"
+    N304 - related to nonexistent "called_once_with"
     """
 
     correct_names = ["assert_any_call", "assert_called_once_with",
-                     "assert_called_with", "assert_has_calls"]
+                     "assert_called_with", "assert_has_calls",
+                     "assert_not_called"]
     ignored_files = ["./tests/unit/test_hacking.py"]
 
     if filename.startswith("./tests") and filename not in ignored_files:
@@ -123,6 +139,11 @@ def check_assert_methods_from_mock(logical_line, physical_line, filename):
                                   "'assertEqual(1, %s.call_count)' "
                                   "or '%s.assert_called_once_with()'"
                                   " instead." % (obj_name, obj_name))
+                elif method_name == "called_once_with":
+                    error_number = "N304"
+                    custom_msg = ("Maybe, you should try to use "
+                                  "'%s.assert_called_once_with()'"
+                                  " instead." % obj_name)
                 else:
                     custom_msg = ("Correct 'assert_*' methods: '%s'."
                                   % "', '".join(correct_names))
@@ -142,7 +163,8 @@ def check_import_of_logging(logical_line, physical_line, filename):
 
     excluded_files = ["./rally/common/logging.py",
                       "./tests/unit/test_logging.py",
-                      "./tests/ci/rally_verify.py"]
+                      "./tests/ci/rally_verify.py",
+                      "./tests/ci/sync_requirements.py"]
 
     forbidden_imports = ["from oslo_log",
                          "import oslo_log",
@@ -156,21 +178,22 @@ def check_import_of_logging(logical_line, physical_line, filename):
 
 
 @skip_ignored_lines
-def no_translate_debug_logs(logical_line, physical_line, filename):
-    """Check for "LOG.debug(_("
-
-    As per our translation policy,
-    https://wiki.openstack.org/wiki/LoggingStandards#Log_Translation
-    we shouldn't translate debug level logs.
-
-    * This check assumes that 'LOG' is a logger.
-    * Use filename so we can start enforcing this in specific folders instead
-      of needing to do so all at once.
+def check_import_of_config(logical_line, physical_line, filename):
+    """Check correctness import of config module
 
     N311
     """
-    if logical_line.startswith("LOG.debug(_("):
-        yield(0, "N311 Don't translate debug level logs")
+
+    excluded_files = ["./rally/common/cfg.py"]
+
+    forbidden_imports = ["from oslo_config",
+                         "import oslo_config"]
+
+    if filename not in excluded_files:
+        for forbidden_import in forbidden_imports:
+            if logical_line.startswith(forbidden_import):
+                yield (0, "N311 Wrong module for config is imported. Please "
+                          "use `rally.common.cfg` instead.")
 
 
 @skip_ignored_lines
@@ -265,6 +288,37 @@ def assert_equal_in(logical_line, physical_line, filename):
                   "collection contents.")
 
 
+@skip_ignored_lines
+def assert_not_equal_none(logical_line, physical_line, filename):
+    """Check for assertNotEqual(A, None) or assertEqual(None, A) sentences
+
+    N325
+    """
+    res = (re_assert_not_equal_start_with_none.search(logical_line) or
+           re_assert_not_equal_end_with_none.search(logical_line))
+    if res:
+        yield (0, "N325 assertNotEqual(A, None) or assertNotEqual(None, A) "
+                  "sentences not allowed, you should use assertIsNotNone(A) "
+                  "instead.")
+
+
+@skip_ignored_lines
+def assert_equal_true_or_false(logical_line, physical_line, filename):
+    """Check for assertEqual(A, True/False) sentences
+
+    Check for assertEqual(A, True/False) sentences or
+    assertEqual(True/False, A)
+
+    N326
+    """
+    res = (re_assert_equal_end_with_true_or_false.search(logical_line) or
+           re_assert_equal_start_with_true_or_false.search(logical_line))
+    if res:
+        yield (0, "N326 assertEqual(A, True/False) or "
+                  "assertEqual(True/False, A) sentences not allowed,"
+                  "you should use assertTrue(A) or assertFalse(A) instead.")
+
+
 @skip_ignored_lines
 def check_no_direct_rally_objects_import(logical_line, physical_line,
                                          filename):
@@ -465,6 +519,39 @@ def check_raises(physical_line, filename):
                        "in docstrings.")
 
 
+@skip_ignored_lines
+def check_old_type_class(logical_line, physical_line, filename):
+    """Use new-style Python classes
+
+    N355
+    """
+
+    if re_old_type_class.search(logical_line):
+        yield (0, "N355 This class does not inherit from anything and thus "
+                  "will be an old-style class by default. Try to inherit from "
+                  "``object`` or another new-style class.")
+
+
+@skip_ignored_lines
+def check_datetime_alias(logical_line, physical_line, filename):
+    """Ensure using ``dt`` as alias for ``datetime``
+
+    N356
+    """
+    if re_datetime_alias.search(logical_line):
+        yield (0, "N356 Please use ``dt`` as alias for ``datetime``.")
+
+
+@skip_ignored_lines
+def check_no_six_iteritems(logical_line, physical_line, filename):
+    """Check no six.iteritems
+
+    N357
+    """
+    if re.search(r"\six.iteritems\(\)", logical_line):
+        yield (0, "N357 Use dict.items() instead of six.iteritems()")
+
+
 @skip_ignored_lines
 def check_db_imports_in_cli(logical_line, physical_line, filename):
     """Ensure that CLI modules do not use ``rally.common.db``
@@ -472,7 +559,7 @@ def check_db_imports_in_cli(logical_line, physical_line, filename):
     N360
     """
     if (not filename.startswith("./rally/cli")
-            or filename == "./rally/cli/manage.py"):
+            or filename == "./rally/cli/commands/db.py"):
         return
     if re_db_import.search(logical_line):
         yield (0, "N360 CLI modules do not allow to work with "
@@ -485,24 +572,50 @@ def check_objects_imports_in_cli(logical_line, physical_line, filename):
 
     N361
     """
-    if (not filename.startswith("./rally/cli")
-            or filename == "./rally/cli/commands/show.py"):
+    if not filename.startswith("./rally/cli"):
         return
     if re_objects_import.search(logical_line):
-        yield (0, "N360 CLI modules do not allow to work with "
+        yield (0, "N361 CLI modules do not allow to work with "
                   "`rally.common.objects``.")
 
 
+@skip_ignored_lines
+def check_log_warn(logical_line, physical_line, filename):
+    if re_log_warn.search(logical_line):
+        yield(0, "N313 LOG.warn is deprecated, please use LOG.warning")
+
+
+@skip_ignored_lines
+def check_opts_import_path(logical_line, physical_line, filename):
+    """Ensure that we load opts from correct paths only
+
+     N342
+     """
+    excluded_files = ["./rally/task/engine.py",
+                      "./rally/task/context.py",
+                      "./rally/task/scenario.py",
+                      "./rally/common/opts.py"]
+    forbidden_methods = [".register_opts("]
+
+    if filename not in excluded_files:
+        for forbidden_method in forbidden_methods:
+            if logical_line.find(forbidden_method) != -1:
+                yield (0, "N342 All options should be loaded from correct "
+                          "paths only. For 'openstack' "
+                          "its './rally/plugin/openstack/cfg'")
+
+
 def factory(register):
     register(check_assert_methods_from_mock)
     register(check_import_of_logging)
-    register(no_translate_debug_logs)
+    register(check_import_of_config)
     register(no_use_conf_debug_check)
     register(assert_true_instance)
     register(assert_equal_type)
     register(assert_equal_none)
     register(assert_true_or_false_with_in)
     register(assert_equal_in)
+    register(assert_equal_true_or_false)
     register(check_no_direct_rally_objects_import)
     register(check_no_oslo_deprecated_import)
     register(check_quotes)
@@ -510,5 +623,10 @@ def factory(register):
     register(check_dict_formatting_in_string)
     register(check_using_unicode)
     register(check_raises)
+    register(check_datetime_alias)
     register(check_db_imports_in_cli)
     register(check_objects_imports_in_cli)
+    register(check_old_type_class)
+    register(check_no_six_iteritems)
+    register(check_log_warn)
+    register(check_opts_import_path)
