@@ -26,18 +26,5 @@ class CmdManageTestCase(test.TestCase):
     @mock.patch("rally.cli.manage.cliutils")
     def test_main(self, mock_cliutils):
         manage.main()
-        categories = {"db": manage.DBCommands}
+        categories = {"db": manage.db.DBCommands}
         mock_cliutils.run.assert_called_once_with(sys.argv, categories)
-
-
-class DBCommandsTestCase(test.TestCase):
-
-    def setUp(self):
-        super(DBCommandsTestCase, self).setUp()
-        self.db_commands = manage.DBCommands()
-
-    @mock.patch("rally.cli.manage.db")
-    def test_recreate(self, mock_db):
-        self.db_commands.recreate()
-        calls = [mock.call.db_drop(), mock.call.db_create()]
-        self.assertEqual(calls, mock_db.mock_calls)
