@@ -14,10 +14,13 @@
 #    under the License.
 
 import os
+import socket
 
+import ddt
 import mock
 
 from rally.common import sshutils
+from rally import exceptions
 from tests.unit import test
 
 
@@ -60,7 +63,7 @@ class SSHTestCase(test.TestCase):
         dss = mock_paramiko.dsskey.DSSKey
         rsa.from_private_key.side_effect = mock_paramiko.SSHException
         dss.from_private_key.side_effect = mock_paramiko.SSHException
-        self.assertRaises(sshutils.SSHError, self.ssh._get_pkey, "key")
+        self.assertRaises(exceptions.SSHError, self.ssh._get_pkey, "key")
 
     @mock.patch("rally.common.sshutils.six.moves.StringIO")
     @mock.patch("rally.common.sshutils.paramiko")
@@ -73,7 +76,7 @@ class SSHTestCase(test.TestCase):
         key = self.ssh._get_pkey("key")
         dss_calls = mock_paramiko.dsskey.DSSKey.from_private_key.mock_calls
         self.assertEqual([mock.call("string_key")], dss_calls)
-        self.assertEqual(key, "dss_key")
+        self.assertEqual("dss_key", key)
         mock_string_io.assert_called_once_with("key")
 
     @mock.patch("rally.common.sshutils.six.moves.StringIO")
@@ -87,7 +90,7 @@ class SSHTestCase(test.TestCase):
         key = self.ssh._get_pkey("key")
         rsa_calls = mock_paramiko.rsakey.RSAKey.from_private_key.mock_calls
         self.assertEqual([mock.call("string_key")], rsa_calls)
-        self.assertEqual(key, "rsa_key")
+        self.assertEqual("rsa_key", key)
         mock_string_io.assert_called_once_with("key")
 
     @mock.patch("rally.common.sshutils.SSH._get_pkey")
@@ -121,7 +124,8 @@ class SSHTestCase(test.TestCase):
         mock_string_io.side_effect = stdio = [mock.Mock(), mock.Mock()]
         stdio[0].read.return_value = "stdout fake data"
         stdio[1].read.return_value = "stderr fake data"
-        with mock.patch.object(self.ssh, "run", return_value=0) as mock_run:
+        with mock.patch.object(self.ssh, "run") as mock_run:
+            mock_run.return_value = (0, None)
             status, stdout, stderr = self.ssh.execute("cmd",
                                                       stdin="fake_stdin",
                                                       timeout=43)
@@ -135,22 +139,23 @@ class SSHTestCase(test.TestCase):
     @mock.patch("rally.common.sshutils.time")
     def test_wait_timeout(self, mock_time):
         mock_time.time.side_effect = [1, 50, 150]
-        self.ssh.execute = mock.Mock(side_effect=[sshutils.SSHError,
-                                                  sshutils.SSHError,
+        self.ssh.execute = mock.Mock(side_effect=[exceptions.SSHError,
+                                                  exceptions.SSHError,
                                                   0])
-        self.assertRaises(sshutils.SSHTimeout, self.ssh.wait)
+        self.assertRaises(exceptions.SSHTimeout, self.ssh.wait)
         self.assertEqual([mock.call("uname")] * 2, self.ssh.execute.mock_calls)
 
     @mock.patch("rally.common.sshutils.time")
     def test_wait(self, mock_time):
         mock_time.time.side_effect = [1, 50, 100]
-        self.ssh.execute = mock.Mock(side_effect=[sshutils.SSHError,
-                                                  sshutils.SSHError,
+        self.ssh.execute = mock.Mock(side_effect=[exceptions.SSHError,
+                                                  exceptions.SSHError,
                                                   0])
         self.ssh.wait()
         self.assertEqual([mock.call("uname")] * 3, self.ssh.execute.mock_calls)
 
 
+@ddt.ddt
 class SSHRunTestCase(test.TestCase):
     """Test SSH.run method in different aspects.
 
@@ -206,14 +211,16 @@ class SSHRunTestCase(test.TestCase):
     @mock.patch("rally.common.sshutils.select")
     def test_run(self, mock_select):
         mock_select.select.return_value = ([], [], [])
-        self.assertEqual(0, self.ssh.run("cmd"))
+        exit_stat, data = self.ssh.run("cmd")
+        self.assertEqual(0, exit_stat)
 
     @mock.patch("rally.common.sshutils.select")
     def test_run_nonzero_status(self, mock_select):
         mock_select.select.return_value = ([], [], [])
         self.fake_session.recv_exit_status.return_value = 1
-        self.assertRaises(sshutils.SSHError, self.ssh.run, "cmd")
-        self.assertEqual(1, self.ssh.run("cmd", raise_on_error=False))
+        self.assertRaises(exceptions.SSHError, self.ssh.run, "cmd")
+        exit_stat, data = self.ssh.run("cmd", raise_on_error=False)
+        self.assertEqual(1, exit_stat)
 
     @mock.patch("rally.common.sshutils.select")
     def test_run_stdout(self, mock_select):
@@ -261,7 +268,7 @@ class SSHRunTestCase(test.TestCase):
     def test_run_select_error(self, mock_select):
         self.fake_session.exit_status_ready.return_value = False
         mock_select.select.return_value = ([], [], [True])
-        self.assertRaises(sshutils.SSHError, self.ssh.run, "cmd")
+        self.assertRaises(exceptions.SSHError, self.ssh.run, "cmd")
 
     @mock.patch("rally.common.sshutils.time")
     @mock.patch("rally.common.sshutils.select")
@@ -269,7 +276,7 @@ class SSHRunTestCase(test.TestCase):
         mock_time.time.side_effect = [1, 3700]
         mock_select.select.return_value = ([], [], [])
         self.fake_session.exit_status_ready.return_value = False
-        self.assertRaises(sshutils.SSHTimeout, self.ssh.run, "cmd")
+        self.assertRaises(exceptions.SSHTimeout, self.ssh.run, "cmd")
 
     @mock.patch("rally.common.sshutils.open", create=True)
     def test__put_file_shell(self, mock_open):
@@ -304,9 +311,9 @@ class SSHRunTestCase(test.TestCase):
         sftp.chmod.assert_called_once_with("remotefile", 0o753)
         sftp.__exit__.assert_called_once_with(None, None, None)
 
-    def test_put_file(self):
-        self.ssh._put_file_sftp = mock.Mock(
-            side_effect=sshutils.paramiko.SSHException())
+    @ddt.data(sshutils.paramiko.SSHException, socket.error)
+    def test_put_file(self, exc):
+        self.ssh._put_file_sftp = mock.Mock(side_effect=exc())
         self.ssh._put_file_shell = mock.Mock()
 
         self.ssh.put_file("foo", "bar", 42)
