@@ -13,6 +13,7 @@
 #    under the License.
 
 from __future__ import print_function
+import collections
 import string
 import sys
 import threading
@@ -32,7 +33,7 @@ class ImmutableMixinTestCase(test.TestCase):
 
     def test_without_base_values(self):
         im = utils.ImmutableMixin()
-        self.assertRaises(exceptions.ImmutableException,
+        self.assertRaises(AttributeError,
                           im.__setattr__, "test", "test")
 
     def test_with_base_values(self):
@@ -43,9 +44,9 @@ class ImmutableMixinTestCase(test.TestCase):
                 super(A, self).__init__()
 
         a = A("test")
-        self.assertRaises(exceptions.ImmutableException,
+        self.assertRaises(AttributeError,
                           a.__setattr__, "abc", "test")
-        self.assertEqual(a.test, "test")
+        self.assertEqual("test", a.test)
 
 
 class EnumMixinTestCase(test.TestCase):
@@ -57,7 +58,7 @@ class EnumMixinTestCase(test.TestCase):
             b = 20
             CC = "2000"
 
-        self.assertEqual(set(list(Foo())), set([10, 20, "2000"]))
+        self.assertEqual(set([10, 20, "2000"]), set(list(Foo())))
 
     def test_with_underscore(self):
 
@@ -66,7 +67,7 @@ class EnumMixinTestCase(test.TestCase):
             b = 20
             _CC = "2000"
 
-        self.assertEqual(set(list(Foo())), set([10, 20]))
+        self.assertEqual(set([10, 20]), set(list(Foo())))
 
 
 class StdIOCaptureTestCase(test.TestCase):
@@ -78,8 +79,8 @@ class StdIOCaptureTestCase(test.TestCase):
             for msg in messages:
                 print(msg)
 
-        self.assertEqual(out.getvalue().rstrip("\n").split("\n"), messages)
-        self.assertEqual(stdout, sys.stdout)
+        self.assertEqual(messages, out.getvalue().rstrip("\n").split("\n"))
+        self.assertEqual(sys.stdout, stdout)
 
     def test_stderr_capture(self):
         stderr = sys.stderr
@@ -88,8 +89,8 @@ class StdIOCaptureTestCase(test.TestCase):
             for msg in messages:
                 print(msg, file=sys.stderr)
 
-        self.assertEqual(err.getvalue().rstrip("\n").split("\n"), messages)
-        self.assertEqual(stderr, sys.stderr)
+        self.assertEqual(messages, err.getvalue().rstrip("\n").split("\n"))
+        self.assertEqual(sys.stderr, stderr)
 
 
 class TimerTestCase(test.TestCase):
@@ -104,6 +105,8 @@ class TimerTestCase(test.TestCase):
                 mock_time.time = mock.MagicMock(return_value=end_time)
 
         self.assertIsNone(timer.error)
+        self.assertEqual(start_time, timer.timestamp())
+        self.assertEqual(end_time, timer.finish_timestamp())
         self.assertEqual(end_time - start_time, timer.duration())
 
     def test_timer_exception(self):
@@ -138,9 +141,9 @@ class FirstIndexTestCase(test.TestCase):
 
     def test_list_with_existing_matching_element(self):
         lst = [1, 3, 5, 7]
-        self.assertEqual(utils.first_index(lst, lambda e: e == 1), 0)
-        self.assertEqual(utils.first_index(lst, lambda e: e == 5), 2)
-        self.assertEqual(utils.first_index(lst, lambda e: e == 7), 3)
+        self.assertEqual(0, utils.first_index(lst, lambda e: e == 1))
+        self.assertEqual(2, utils.first_index(lst, lambda e: e == 5))
+        self.assertEqual(3, utils.first_index(lst, lambda e: e == 7))
 
     def test_list_with_non_existing_matching_element(self):
         lst = [1, 3, 5, 7]
@@ -239,7 +242,7 @@ class RandomNameTestCase(test.TestCase):
         generator = FakeNameGenerator()
 
         mock_choice.side_effect = iter("blarglesdweebled")
-        self.assertEqual(generator.generate_random_name(), expected)
+        self.assertEqual(expected, generator.generate_random_name())
 
         class FakeNameGenerator(utils.RandomNameGeneratorMixin):
             RESOURCE_NAME_FORMAT = fmt
@@ -248,7 +251,7 @@ class RandomNameTestCase(test.TestCase):
         generator = FakeNameGenerator()
 
         mock_choice.side_effect = iter("blarglesdweebled")
-        self.assertEqual(generator.generate_random_name(), expected)
+        self.assertEqual(expected, generator.generate_random_name())
 
     def test_generate_random_name_bogus_name_format(self):
         class FakeNameGenerator(utils.RandomNameGeneratorMixin):
@@ -390,13 +393,19 @@ class RandomNameTestCase(test.TestCase):
 
         names = [generator.generate_random_name() for i in range(100)]
         task_id_parts = set([n.split("_")[0] for n in names])
-        self.assertEqual(len(task_id_parts), 1)
+        self.assertEqual(1, len(task_id_parts))
 
         generator.task = {"uuid": "bogus! task! id!"}
 
         names = [generator.generate_random_name() for i in range(100)]
         task_id_parts = set([n.split("_")[0] for n in names])
-        self.assertEqual(len(task_id_parts), 1)
+        self.assertEqual(1, len(task_id_parts))
+
+    def test_make_name_matcher(self):
+        matcher = utils.make_name_matcher("foo", "bar")
+        self.assertTrue(matcher.name_matches_object("foo", task_id="task"))
+        self.assertTrue(matcher.name_matches_object("bar", task_id="task"))
+        self.assertFalse(matcher.name_matches_object("foo1", task_id="task"))
 
 
 @ddt.ddt
@@ -448,7 +457,7 @@ class MergeTestCase(test.TestCase):
         in_iters = [iter(src) for src in sources]
 
         out = list(utils.merge(10, *in_iters))
-        self.assertEqual(out, expected_output)
+        self.assertEqual(expected_output, out)
 
 
 class TimeoutThreadTestCase(test.TestCase):
@@ -472,7 +481,7 @@ class TimeoutThreadTestCase(test.TestCase):
         )
         test_thread.start()
         start_time = time.time()
-        queue.put((test_thread.ident, start_time + 1))
+        queue.put((test_thread, start_time + 1))
         killer_thread.start()
         test_thread.join()
         end_time = time.time()
@@ -482,5 +491,275 @@ class TimeoutThreadTestCase(test.TestCase):
         # NOTE(sskripnick): Killing thread with PyThreadState_SetAsyncExc
         # works with sinificant delay. Make sure this delay is less
         # than 10 seconds.
-        self.assertTrue(time_elapsed < 11,
+        self.assertLess(time_elapsed, 11,
                         "Thread killed too late (%s seconds)" % time_elapsed)
+
+
+class LockedDictTestCase(test.TestCase):
+
+    def test_init_unlock_and_update(self):
+        def setitem(obj, key, value):
+            obj[key] = value
+
+        def delitem(obj, key):
+            del obj[key]
+
+        d = utils.LockedDict()
+        self.assertIsInstance(d, dict)
+        self.assertEqual({}, d)
+
+        d = utils.LockedDict(foo="bar", spam={"a": ["b", {"c": "d"}]})
+        self.assertEqual({"foo": "bar", "spam": {"a": ("b", {"c": "d"})}}, d)
+        self.assertIsInstance(d["spam"], utils.LockedDict)
+        self.assertIsInstance(d["spam"]["a"][1], utils.LockedDict)
+        self.assertRaises(RuntimeError, setitem, d, 123, 456)
+        self.assertRaises(RuntimeError, delitem, d, "foo")
+        self.assertRaises(RuntimeError, setitem, d["spam"]["a"][1], 123, 456)
+        self.assertRaises(RuntimeError, delitem, d["spam"]["a"][1], "c")
+        self.assertRaises(RuntimeError, d.update, {123: 456})
+        self.assertRaises(RuntimeError, d.setdefault, 123, 456)
+        self.assertRaises(RuntimeError, d.pop, "foo")
+        self.assertRaises(RuntimeError, d.popitem)
+        self.assertRaises(RuntimeError, d.clear)
+        self.assertEqual({"foo": "bar", "spam": {"a": ("b", {"c": "d"})}}, d)
+
+        with d.unlocked():
+            d["spam"] = 42
+            self.assertEqual({"foo": "bar", "spam": 42}, d)
+            d.clear()
+            self.assertEqual({}, d)
+            d.setdefault("foo", 42)
+            d.update({"bar": 24})
+            self.assertEqual({"foo": 42, "bar": 24}, d)
+            self.assertEqual(24, d.pop("bar"))
+            self.assertEqual(("foo", 42), d.popitem())
+            d[123] = 456
+
+        self.assertEqual({123: 456}, d)
+
+        self.assertRaises(RuntimeError, setitem, d, 123, 456)
+        self.assertRaises(RuntimeError, delitem, d, "foo")
+
+    @mock.patch("rally.common.utils.copy.deepcopy")
+    def test___deepcopy__(self, mock_deepcopy):
+        mock_deepcopy.side_effect = lambda *args, **kw: (args, kw)
+        d = utils.LockedDict(foo="bar", spam={"a": ["b", {"c": "d"}]})
+        args, kw = d.__deepcopy__()
+        self.assertEqual({"memo": None}, kw)
+        self.assertEqual(({"foo": "bar", "spam": {"a": ("b", {"c": "d"})}},),
+                         args)
+        self.assertEqual(dict, type(args[0]))
+        self.assertEqual(dict, type(args[0]["spam"]))
+        self.assertEqual(dict, type(args[0]["spam"]["a"][1]))
+
+        mock_deepcopy.reset_mock()
+        args, kw = d.__deepcopy__("foo_memo")
+        self.assertEqual(({"foo": "bar", "spam": {"a": ("b", {"c": "d"})}},),
+                         args)
+        self.assertEqual({"memo": "foo_memo"}, kw)
+
+
+class DequeAsQueueTestCase(test.TestCase):
+
+    def setUp(self):
+        super(DequeAsQueueTestCase, self).setUp()
+        self.deque = collections.deque()
+        self.deque_as_queue = utils.DequeAsQueue(self.deque)
+
+    def test_qsize(self):
+        self.assertEqual(0, self.deque_as_queue.qsize())
+        self.deque.append(10)
+        self.assertEqual(1, self.deque_as_queue.qsize())
+
+    def test_put(self):
+        self.deque_as_queue.put(10)
+        self.assertEqual(10, self.deque.popleft())
+
+    def test_get(self):
+        self.deque.append(33)
+        self.assertEqual(33, self.deque_as_queue.get())
+
+    def test_empty(self):
+        self.assertFalse(self.deque_as_queue.empty())
+        self.deque.append(10)
+        self.assertTrue(self.deque_as_queue.empty())
+
+
+class StopwatchTestCase(test.TestCase):
+
+    @mock.patch("rally.common.utils.interruptable_sleep")
+    @mock.patch("rally.common.utils.time")
+    def test_stopwatch(self, mock_time, mock_interruptable_sleep):
+        mock_time.time.side_effect = [0, 0, 1, 2, 3]
+
+        sw = utils.Stopwatch()
+        sw.start()
+        sw.sleep(1)
+        sw.sleep(2)
+        sw.sleep(3)
+
+        mock_interruptable_sleep.assert_has_calls([
+            mock.call(1),
+            mock.call(1),
+            mock.call(1),
+        ])
+
+    @mock.patch("rally.common.utils.interruptable_sleep")
+    @mock.patch("rally.common.utils.time")
+    def test_no_sleep(self, mock_time, mock_interruptable_sleep):
+        mock_time.time.side_effect = [0, 1]
+
+        sw = utils.Stopwatch()
+        sw.start()
+        sw.sleep(1)
+
+        self.assertFalse(mock_interruptable_sleep.called)
+
+    @mock.patch("rally.common.utils.time")
+    def test_stopwatch_with_event(self, mock_time):
+        mock_time.time.side_effect = [0, 0, 1, 2, 3]
+        event = mock.Mock(spec=threading.Event)()
+
+        sw = utils.Stopwatch(stop_event=event)
+        sw.start()
+        sw.sleep(1)
+        sw.sleep(2)
+        sw.sleep(3)
+
+        event.wait.assert_has_calls([
+            mock.call(1),
+            mock.call(1),
+            mock.call(1),
+        ])
+
+
+class BackupTestCase(test.TestCase):
+    def setUp(self):
+        super(BackupTestCase, self).setUp()
+        p = mock.patch("rally.common.utils.os.mkdir")
+        self.mock_mkdir = p.start()
+        self.addCleanup(p.stop)
+
+    @mock.patch("rally.common.utils.os.path.exists")
+    @mock.patch("rally.common.utils.uuid")
+    def test_generate_random_path(self, mock_uuid, mock_exists):
+        mock_exists.side_effect = lambda a: "exist" in a
+        mock_uuid.uuid4.side_effect = ("exist", "foo")
+
+        self.assertEqual("/some/foo", utils.generate_random_path("/some"))
+
+        mock_exists.assert_has_calls((
+            mock.call("/some/exist"),
+            mock.call("/some/foo"),
+        ))
+
+    @mock.patch("rally.common.utils.generate_random_path")
+    def test___init__(self, mock_generate_random_path):
+        utils.BackupHelper()
+
+        mock_generate_random_path.assert_called_once_with()
+        self.mock_mkdir.assert_called_once_with(
+            mock_generate_random_path.return_value)
+
+    @mock.patch("rally.common.utils.generate_random_path")
+    @mock.patch("rally.common.utils.shutil.copytree")
+    def test_backup(self, mock_copytree, mock_generate_random_path):
+        backup_dir = "another_dir"
+        mock_generate_random_path.side_effect = ("base_tmp_dir", backup_dir)
+
+        bh = utils.BackupHelper()
+
+        path = "some_path"
+        bh.backup(path)
+        mock_copytree.assert_called_once_with(path, backup_dir, symlinks=True)
+        self.assertEqual(backup_dir, bh._stored_data[path])
+        mock_copytree.reset_mock()
+
+        self.assertRaises(exceptions.RallyException, bh.backup, path)
+        self.assertFalse(mock_copytree.called)
+
+    @mock.patch("rally.common.utils.BackupHelper.rollback")
+    @mock.patch("rally.common.utils.generate_random_path")
+    @mock.patch("rally.common.utils.shutil.copytree")
+    def test_backup_failed_while_copy(self, mock_copytree,
+                                      mock_generate_random_path,
+                                      mock_backup_helper_rollback):
+        backup_dir = "another_dir"
+        mock_generate_random_path.side_effect = ("base_tmp_dir", backup_dir)
+        mock_copytree.side_effect = RuntimeError
+
+        bh = utils.BackupHelper()
+
+        path = "some_path"
+        self.assertRaises(RuntimeError, bh.backup, path)
+        mock_copytree.assert_called_once_with(path, backup_dir, symlinks=True)
+        self.assertTrue(not bh._stored_data)
+        mock_backup_helper_rollback.assert_called_once_with()
+
+    @mock.patch("rally.common.utils.BackupHelper.backup")
+    def test_call(self, mock_backup_helper_backup):
+        path = "/some/path"
+
+        bh = utils.BackupHelper()
+
+        self.assertEqual(bh, bh(path))
+        mock_backup_helper_backup.assert_called_once_with(path)
+
+    @mock.patch("rally.common.utils."
+                "os.path.exists", side_effect=(False, True, True))
+    @mock.patch("rally.common.utils.shutil.rmtree")
+    def test___del__(self, mock_rmtree, mock_exists):
+        paths = {"original_path": "/tmp/backup_of_something",
+                 "another_path": "/tmp/backup_of_another_thing"}
+        bh = utils.BackupHelper()
+        bh._stored_data = paths
+
+        del bh
+
+        self.assertEqual([mock.call(p) for p in paths.values()],
+                         mock_rmtree.call_args_list)
+
+    @mock.patch("rally.common.utils.os.path.exists", side_effect=(False, True))
+    @mock.patch("rally.common.utils.shutil.rmtree")
+    @mock.patch("rally.common.utils.shutil.copytree")
+    def test_rollback(self, mock_copytree, mock_rmtree, mock_exists):
+
+        bh = utils.BackupHelper()
+
+        original_path = "/some/original/path"
+        tmp_path = "/temporary/location/for/backup"
+        path = {original_path: tmp_path}
+        bh._stored_data = path
+
+        rollback_method = mock.MagicMock()
+        args = (1, 2, 3)
+        kwargs = {"arg1": "value"}
+        bh.add_rollback_action(rollback_method, *args, **kwargs)
+
+        bh.rollback()
+
+        mock_rmtree.assert_called_once_with(original_path)
+        mock_copytree.assert_called_once_with(tmp_path, original_path,
+                                              symlinks=True)
+        self.assertTrue(not bh._stored_data)
+        rollback_method.assert_called_once_with(*args, **kwargs)
+
+    @mock.patch("rally.common.utils.BackupHelper.rollback")
+    def test_context_manager(self, mock_backup_helper_rollback):
+        bh = utils.BackupHelper()
+        with bh:
+            pass
+        self.assertFalse(mock_backup_helper_rollback.called)
+
+        bh = utils.BackupHelper()
+        try:
+            with bh:
+                raise RuntimeError()
+        except RuntimeError:
+            # it is expected behaviour
+            pass
+        else:
+            self.fail("BackupHelper context manager should not hide "
+                      "an exception")
+
+        self.assertTrue(mock_backup_helper_rollback.called)
