@@ -13,7 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import jsonschema
+import ddt
 import mock
 
 from rally.plugins.common.runners import constant
@@ -26,6 +26,7 @@ RUNNERS_BASE = "rally.task.runner."
 RUNNERS = "rally.plugins.common.runners."
 
 
+@ddt.ddt
 class ConstantScenarioRunnerTestCase(test.TestCase):
 
     def setUp(self):
@@ -37,14 +38,32 @@ class ConstantScenarioRunnerTestCase(test.TestCase):
         self.args = {"a": 1}
         self.task = mock.MagicMock()
 
-    def test_validate(self):
-        constant.ConstantScenarioRunner.validate(self.config)
+    @ddt.data(({"times": 4,
+                "concurrency": 2,
+                "timeout": 2,
+                "max_cpu_count": 2}, True),
+              ({"times": 4,
+                "concurrency": 5,
+                "timeout": 2,
+                "max_cpu_count": 2}, False),
+              ({"foo": "bar"}, False))
+    @ddt.unpack
+    def test_validate(self, config, valid):
+        results = runner.ScenarioRunner.validate(
+            "constant", None, None, config)
+        if valid:
+            self.assertEqual([], results)
+        else:
+            self.assertGreater(len(results), 0)
 
-    def test_validate_failed(self):
-        self.config["new_key"] = "should fail"
-        self.assertRaises(jsonschema.ValidationError,
-                          runner.ScenarioRunner.validate,
-                          self.config)
+    @mock.patch(RUNNERS + "constant.runner")
+    def test__run_scenario_once_with_unpack_args(self, mock_runner):
+        result = constant._run_scenario_once_with_unpack_args(
+            ("FOO", ("BAR", "QUUZ")))
+
+        self.assertEqual(result, mock_runner._run_scenario_once.return_value)
+        mock_runner._run_scenario_once.assert_called_once_with(
+            "FOO", ("BAR", "QUUZ"))
 
     @mock.patch(RUNNERS + "constant.time")
     @mock.patch(RUNNERS + "constant.threading.Thread")
@@ -60,6 +79,8 @@ class ConstantScenarioRunnerTestCase(test.TestCase):
         mock_event = mock.MagicMock(
             is_set=mock.MagicMock(return_value=False))
 
+        mock_event_queue = mock.MagicMock()
+
         times = 4
 
         fake_ram_int = iter(range(10))
@@ -69,8 +90,8 @@ class ConstantScenarioRunnerTestCase(test.TestCase):
         info = {"processes_to_start": 1, "processes_counter": 1}
 
         constant._worker_process(mock_queue, fake_ram_int, 1, 2, times,
-                                 context, "Dummy", "dummy", (), mock_event,
-                                 info)
+                                 context, "Dummy", "dummy", (),
+                                 mock_event_queue, mock_event, info)
 
         self.assertEqual(times + 1, mock_thread.call_count)
         self.assertEqual(times + 1, mock_thread_instance.start.call_count)
@@ -81,24 +102,28 @@ class ConstantScenarioRunnerTestCase(test.TestCase):
         self.assertEqual(times, mock_runner._get_scenario_context.call_count)
 
         for i in range(times):
-            scenario_context = mock_runner._get_scenario_context(context)
-            call = mock.call(args=(mock_queue,
-                                   (i, "Dummy", "dummy",
-                                    scenario_context, ())),
-                             target=mock_runner._worker_thread)
+            scenario_context = mock_runner._get_scenario_context(i, context)
+            call = mock.call(
+                args=(mock_queue, "Dummy", "dummy", scenario_context, (),
+                      mock_event_queue),
+                target=mock_runner._worker_thread,
+            )
             self.assertIn(call, mock_thread.mock_calls)
 
     @mock.patch(RUNNERS_BASE + "_run_scenario_once")
     def test__worker_thread(self, mock__run_scenario_once):
         mock_queue = mock.MagicMock()
 
-        args = ("some_args",)
+        mock_event_queue = mock.MagicMock()
 
-        runner._worker_thread(mock_queue, args)
+        args = ("fake_cls", "fake_method_name", "fake_context_obj", {},
+                mock_event_queue)
+
+        runner._worker_thread(mock_queue, *args)
 
         self.assertEqual(1, mock_queue.put.call_count)
 
-        expected_calls = [mock.call(("some_args",))]
+        expected_calls = [mock.call(*args)]
         self.assertEqual(expected_calls, mock__run_scenario_once.mock_calls)
 
     def test__run_scenario(self):
@@ -106,20 +131,20 @@ class ConstantScenarioRunnerTestCase(test.TestCase):
 
         runner_obj._run_scenario(
             fakes.FakeScenario, "do_it", self.context, self.args)
-        self.assertEqual(len(runner_obj.result_queue), self.config["times"])
+        self.assertEqual(self.config["times"], len(runner_obj.result_queue))
         for result_batch in runner_obj.result_queue:
             for result in result_batch:
-                self.assertIsNotNone(runner.ScenarioRunnerResult(result))
+                self.assertIsNotNone(result)
 
     def test__run_scenario_exception(self):
         runner_obj = constant.ConstantScenarioRunner(self.task, self.config)
 
         runner_obj._run_scenario(fakes.FakeScenario, "something_went_wrong",
                                  self.context, self.args)
-        self.assertEqual(len(runner_obj.result_queue), self.config["times"])
+        self.assertEqual(self.config["times"], len(runner_obj.result_queue))
         for result_batch in runner_obj.result_queue:
             for result in result_batch:
-                self.assertIsNotNone(runner.ScenarioRunnerResult(result))
+                self.assertIsNotNone(result)
         self.assertIn("error", runner_obj.result_queue[0][0])
 
     def test__run_scenario_aborted(self):
@@ -128,7 +153,7 @@ class ConstantScenarioRunnerTestCase(test.TestCase):
         runner_obj.abort()
         runner_obj._run_scenario(fakes.FakeScenario, "do_it", self.context,
                                  self.args)
-        self.assertEqual(len(runner_obj.result_queue), 0)
+        self.assertEqual(0, len(runner_obj.result_queue))
 
     @mock.patch(RUNNERS + "constant.multiprocessing.Queue")
     @mock.patch(RUNNERS + "constant.multiprocessing.cpu_count")
@@ -224,7 +249,7 @@ class ConstantScenarioRunnerTestCase(test.TestCase):
             self.assertIn(constant._worker_process, args)
             mock__join_processes.assert_called_once_with(
                 mock__create_process_pool.return_value,
-                mock_queue.return_value)
+                mock_queue.return_value, mock_queue.return_value)
 
     def test_abort(self):
         runner_obj = constant.ConstantScenarioRunner(self.task, self.config)
@@ -233,6 +258,7 @@ class ConstantScenarioRunnerTestCase(test.TestCase):
         self.assertTrue(runner_obj.aborted.is_set())
 
 
+@ddt.ddt
 class ConstantForDurationScenarioRunnerTestCase(test.TestCase):
 
     def setUp(self):
@@ -240,56 +266,61 @@ class ConstantForDurationScenarioRunnerTestCase(test.TestCase):
         self.config = {"duration": 0, "concurrency": 2,
                        "timeout": 2, "type": "constant_for_duration"}
         self.context = fakes.FakeContext({"task": {"uuid": "uuid"}}).context
+        self.context["iteration"] = 14
         self.args = {"a": 1}
 
-    def test_validate(self):
-        constant.ConstantForDurationScenarioRunner.validate(self.config)
-
-    def test_validate_failed(self):
-        self.config["times"] = "gagaga"
-        self.assertRaises(jsonschema.ValidationError,
-                          runner.ScenarioRunner.validate,
-                          self.config)
+    @ddt.data(({"duration": 0,
+                "concurrency": 2,
+                "timeout": 2}, True),
+              ({"foo": "bar"}, False))
+    @ddt.unpack
+    def test_validate(self, config, valid):
+        results = runner.ScenarioRunner.validate(
+            "constant_for_duration", None, None, config)
+        if valid:
+            self.assertEqual([], results)
+        else:
+            self.assertGreater(len(results), 0)
 
     def test_run_scenario_constantly_for_duration(self):
         runner_obj = constant.ConstantForDurationScenarioRunner(
-            None, self.config)
+            mock.MagicMock(), self.config)
 
         runner_obj._run_scenario(fakes.FakeScenario, "do_it",
                                  self.context, self.args)
         # NOTE(mmorais): when duration is 0, scenario executes exactly 1 time
         expected_times = 1
-        self.assertEqual(len(runner_obj.result_queue), expected_times)
+        self.assertEqual(expected_times, len(runner_obj.result_queue))
         for result_batch in runner_obj.result_queue:
             for result in result_batch:
-                self.assertIsNotNone(runner.ScenarioRunnerResult(result))
+                self.assertIsNotNone(result)
 
     def test_run_scenario_constantly_for_duration_exception(self):
         runner_obj = constant.ConstantForDurationScenarioRunner(
-            None, self.config)
+            mock.MagicMock(), self.config)
 
         runner_obj._run_scenario(fakes.FakeScenario, "something_went_wrong",
                                  self.context, self.args)
         # NOTE(mmorais): when duration is 0, scenario executes exactly 1 time
         expected_times = 1
-        self.assertEqual(len(runner_obj.result_queue), expected_times)
+        self.assertEqual(expected_times, len(runner_obj.result_queue))
         for result_batch in runner_obj.result_queue:
             for result in result_batch:
-                self.assertIsNotNone(runner.ScenarioRunnerResult(result))
+                self.assertIsNotNone(result)
         self.assertIn("error", runner_obj.result_queue[0][0])
 
     def test_run_scenario_constantly_for_duration_timeout(self):
         runner_obj = constant.ConstantForDurationScenarioRunner(
-            None, self.config)
+            mock.MagicMock(), self.config)
 
         runner_obj._run_scenario(fakes.FakeScenario, "raise_timeout",
                                  self.context, self.args)
         # NOTE(mmorais): when duration is 0, scenario executes exactly 1 time
         expected_times = 1
-        self.assertEqual(len(runner_obj.result_queue), expected_times)
+        self.assertEqual(expected_times, len(runner_obj.result_queue))
         for result_batch in runner_obj.result_queue:
             for result in result_batch:
-                self.assertIsNotNone(runner.ScenarioRunnerResult(result))
+                self.assertIsNotNone(result)
         self.assertIn("error", runner_obj.result_queue[0][0])
 
     def test__run_scenario_constantly_aborted(self):
@@ -299,7 +330,7 @@ class ConstantForDurationScenarioRunnerTestCase(test.TestCase):
         runner_obj.abort()
         runner_obj._run_scenario(fakes.FakeScenario, "do_it",
                                  self.context, self.args)
-        self.assertEqual(len(runner_obj.result_queue), 0)
+        self.assertEqual(0, len(runner_obj.result_queue))
 
     def test_abort(self):
         runner_obj = constant.ConstantForDurationScenarioRunner(None,
