@@ -22,23 +22,37 @@ from tests.unit import test
 
 class SerialScenarioRunnerTestCase(test.TestCase):
 
+    @mock.patch("rally.common.utils.DequeAsQueue")
     @mock.patch("rally.task.runner._run_scenario_once")
-    def test__run_scenario(self, mock__run_scenario_once):
+    def test__run_scenario(self, mock__run_scenario_once, mock_deque_as_queue):
         times = 5
-        result = {"duration": 10, "idle_duration": 0, "error": [],
+        result = {"duration": 10., "idle_duration": 0., "error": [],
                   "output": {"additive": [], "complete": []},
-                  "atomic_actions": {},
-                  "timestamp": 1}
+                  "atomic_actions": [],
+                  "timestamp": 1.}
         mock__run_scenario_once.return_value = result
+        deque_as_queue_inst = mock_deque_as_queue.return_value
         expected_results = [[result] for i in range(times)]
-
         runner = serial.SerialScenarioRunner(mock.MagicMock(),
                                              {"times": times})
+
         runner._run_scenario(fakes.FakeScenario, "do_it",
                              fakes.FakeContext().context, {})
-        self.assertEqual(len(runner.result_queue), times)
+
+        self.assertEqual(times, len(runner.result_queue))
         results = list(runner.result_queue)
-        self.assertEqual(results, expected_results)
+        self.assertEqual(expected_results, results)
+        expected_calls = []
+        for i in range(times):
+            ctxt = fakes.FakeContext().context
+            ctxt["iteration"] = i + 1
+            ctxt["task"] = mock.ANY
+            expected_calls.append(
+                mock.call(fakes.FakeScenario, "do_it", ctxt, {},
+                          deque_as_queue_inst)
+            )
+        mock__run_scenario_once.assert_has_calls(expected_calls)
+        mock_deque_as_queue.assert_called_once_with(runner.event_queue)
 
     def test__run_scenario_aborted(self):
         runner = serial.SerialScenarioRunner(mock.MagicMock(),
@@ -46,7 +60,7 @@ class SerialScenarioRunnerTestCase(test.TestCase):
         runner.abort()
         runner._run_scenario(fakes.FakeScenario, "do_it",
                              fakes.FakeContext().context, {})
-        self.assertEqual(len(runner.result_queue), 0)
+        self.assertEqual(0, len(runner.result_queue))
 
     def test_abort(self):
         runner = serial.SerialScenarioRunner(mock.MagicMock(),
