@@ -13,10 +13,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import datetime
+import collections
+import datetime as dt
 
+import ddt
 from jsonschema import exceptions as schema_exceptions
 import mock
+import six
 
 from rally import exceptions
 from rally.task import utils
@@ -24,7 +27,7 @@ from tests.unit import fakes
 from tests.unit import test
 
 
-class BenchmarkUtilsTestCase(test.TestCase):
+class TaskUtilsTestCase(test.TestCase):
 
     def test_wait_for_delete(self):
         def update_resource(self):
@@ -84,7 +87,14 @@ class BenchmarkUtilsTestCase(test.TestCase):
         manager = fakes.FakeManager()
         resource = fakes.FakeResource(manager=manager)
         manager._cache(resource)
-        self.assertEqual(get_from_manager(resource), resource)
+        self.assertEqual(resource, get_from_manager(resource))
+
+    def test_get_from_manager_with_uuid_field(self):
+        get_from_manager = utils.get_from_manager()
+        manager = fakes.FakeManager()
+        resource = fakes.FakeResource(manager=manager)
+        manager._cache(resource)
+        self.assertEqual(resource, get_from_manager(resource, id_attr="uuid"))
 
     def test_get_from_manager_in_error_state(self):
         get_from_manager = utils.get_from_manager()
@@ -144,42 +154,6 @@ class BenchmarkUtilsTestCase(test.TestCase):
         self.assertRaises(exceptions.GetResourceFailure,
                           get_from_manager, resource)
 
-    def test_check_service_status(self):
-        class service():
-            def __init__(self, name):
-                self.status = "enabled"
-                self.state = "up"
-                self.name = name
-
-            def __str__(self):
-                return self.name
-
-        client = mock.MagicMock()
-        client.services.list.return_value = [service("nova-compute"),
-                                             service("nova-network"),
-                                             service("glance-api")]
-        ret = utils.check_service_status(client, "nova-network")
-        self.assertTrue(ret)
-        self.assertTrue(client.services.list.called)
-
-    def test_check_service_status_fail(self):
-        class service():
-            def __init__(self, name):
-                self.status = "enabled"
-                self.state = "down"
-                self.name = name
-
-            def __str__(self):
-                return self.name
-
-        client = mock.MagicMock()
-        client.services.list.return_value = [service("nova-compute"),
-                                             service("nova-network"),
-                                             service("glance-api")]
-        ret = utils.check_service_status(client, "nova-network")
-        self.assertFalse(ret)
-        self.assertTrue(client.services.list.called)
-
 
 class WaitForTestCase(test.TestCase):
 
@@ -192,8 +166,8 @@ class WaitForTestCase(test.TestCase):
             seconds=self.load_secs)
 
     def get_fake_checker_delayed(self, **delay):
-        deadline = datetime.datetime.now() + datetime.timedelta(**delay)
-        return lambda obj: datetime.datetime.now() > deadline
+        deadline = dt.datetime.now() + dt.timedelta(**delay)
+        return lambda obj: dt.datetime.now() > deadline
 
     def fake_checker_false(self, obj):
         return False
@@ -228,10 +202,10 @@ class WaitForTestCase(test.TestCase):
             update_resource=self.fake_updater, timeout=self.load_secs,
             check_interval=self.load_secs / 3)
 
-        self.assertEqual(exc.kwargs["resource_name"], "fake_name")
-        self.assertEqual(exc.kwargs["resource_id"], "fake_id")
-        self.assertEqual(exc.kwargs["desired_status"], "fake_new_status")
-        self.assertEqual(exc.kwargs["resource_status"], "FAKE_STALE_STATUS")
+        self.assertEqual("fake_name", exc.kwargs["resource_name"])
+        self.assertEqual("fake_id", exc.kwargs["resource_id"])
+        self.assertEqual("fake_new_status", exc.kwargs["desired_status"])
+        self.assertEqual("FAKE_STALE_STATUS", exc.kwargs["resource_status"])
 
         self.assertIn("FakeResource", str(exc))
         self.assertIn("fake_new_status", str(exc))
@@ -449,6 +423,28 @@ class WaitForStatusTestCase(test.TestCase):
                               mock.call({"status": "still_not_ready"}),
                               mock.call({"status": "almost_ready"})])
 
+    @mock.patch("rally.task.utils.time.sleep")
+    @mock.patch("rally.task.utils.time.time", return_value=1)
+    def test_wait_successful_with_uuid(self, mock_time, mock_sleep):
+        res = {"status": "not_ready"}
+        upd = mock.MagicMock(side_effect=[{"status": "not_ready"},
+                                          {"status": "not_ready_yet"},
+                                          {"status": "still_not_ready"},
+                                          {"status": "almost_ready"},
+                                          {"status": "ready"}])
+        utils.wait_for(resource=res, ready_statuses=["ready"],
+                       update_resource=upd, id_attr="uuid")
+        upd.assert_has_calls([mock.call({"status": "not_ready"},
+                                        id_attr="uuid"),
+                              mock.call({"status": "not_ready"},
+                                        id_attr="uuid"),
+                              mock.call({"status": "not_ready_yet"},
+                                        id_attr="uuid"),
+                              mock.call({"status": "still_not_ready"},
+                                        id_attr="uuid"),
+                              mock.call({"status": "almost_ready"},
+                                        id_attr="uuid")])
+
     @mock.patch("rally.task.utils.time.sleep")
     @mock.patch("rally.task.utils.time.time", return_value=1)
     def test_wait_failure(self, mock_time, mock_sleep):
@@ -484,3 +480,92 @@ class WaitForStatusTestCase(test.TestCase):
                                     check_deletion=True,
                                     update_resource=upd)
         self.assertEqual(res, ret)
+
+    @mock.patch("rally.task.utils.LOG.debug")
+    @mock.patch("rally.task.utils.time.sleep")
+    @mock.patch("rally.task.utils.time.time", side_effect=[1, 2, 3, 4])
+    def test_wait_timeout(self, mock_time, mock_sleep, mock_log_debug):
+        res = {"status": "not_ready"}
+        upd = mock.MagicMock(side_effect=[{"status": "not_ready"},
+                                          {"status": "fail"}])
+        self.assertRaises(exceptions.TimeoutException,
+                          utils.wait_for_status,
+                          resource=res, ready_statuses=["ready"],
+                          update_resource=upd, timeout=2, id_attr="uuid")
+
+
+@ddt.ddt
+class WrapperForAtomicActionsTestCase(test.TestCase):
+
+    def test_dict_atomic(self):
+        atomic_actions = collections.OrderedDict(
+            [("action_1", 1), ("action_2", 2)])
+        atomic_wrapper = utils.WrapperForAtomicActions(atomic_actions, 1)
+        self.assertEqual(1, atomic_wrapper["action_1"])
+        self.assertEqual(2, atomic_wrapper["action_2"])
+        self.assertEqual(atomic_actions.items(),
+                         atomic_wrapper.items())
+        self.assertEqual(1, atomic_wrapper.get("action_1"))
+        self.assertIsNone(atomic_wrapper.get("action_3"))
+        self.assertEqual(2, len(atomic_wrapper))
+        self.assertEqual([{"name": "action_1", "started_at": 1,
+                           "finished_at": 2, "children": []},
+                          {"name": "action_2", "started_at": 2,
+                           "finished_at": 4, "children": []}
+                          ], atomic_wrapper)
+
+    def test_list_atomic(self):
+        atomic_actions = [{"name": "action_1", "started_at": 1,
+                           "finished_at": 2, "children": []},
+                          {"name": "action_2", "started_at": 2,
+                           "finished_at": 4, "children": []}]
+
+        atomic_wrapper = utils.WrapperForAtomicActions(atomic_actions)
+        self.assertEqual(1, atomic_wrapper["action_1"])
+        self.assertEqual(2, atomic_wrapper["action_2"])
+        self.assertEqual(
+            collections.OrderedDict(
+                [("action_1", 1), ("action_2", 2)]).items(),
+            atomic_wrapper.items())
+        self.assertEqual(atomic_actions[0], atomic_wrapper[0])
+        self.assertEqual(atomic_actions[1], atomic_wrapper[1])
+        self.assertEqual(1, atomic_wrapper.get("action_1"))
+        self.assertIsNone(atomic_wrapper.get("action_3"))
+        self.assertEqual(2, len(atomic_wrapper))
+        self.assertEqual(atomic_actions[0], six.next(iter(atomic_wrapper)))
+
+    def test__convert_new_atomic_actions(self):
+        atomic_actions = collections.OrderedDict(
+            [("action_1", 1), ("action_2", 2)])
+        atomic_wrapper = utils.WrapperForAtomicActions(atomic_actions)
+        self.assertEqual(
+            [{"name": "action_1", "started_at": 0,
+              "finished_at": 1, "children": []},
+             {"name": "action_2", "started_at": 1,
+              "finished_at": 3, "children": []}],
+            atomic_wrapper._convert_old_atomic_actions(atomic_actions))
+
+    @ddt.data(
+        {"atomic_actions": [{"name": "some", "started_at": 1.0,
+                             "finished_at": 2.0, "children": []}],
+         "expected": {"some": 1.0}},
+        {"atomic_actions": [{"name": "some", "started_at": 1.0,
+                             "finished_at": 2.0, "children": []},
+                            {"name": "some", "started_at": 2.0,
+                             "finished_at": 3.0, "children": []}],
+         "expected": {"some": 1.0, "some (2)": 1.0}},
+        {"atomic_actions": [{"name": "some", "started_at": 1.0,
+                             "finished_at": 2.0, "children": []},
+                            {"name": "some", "started_at": 2.0,
+                             "finished_at": 3.0, "children": []},
+                            {"name": "some", "started_at": 3.0,
+                             "finished_at": 4.0, "children": []}
+                            ],
+         "expected": {"some": 1.0, "some (2)": 1.0, "some (3)": 1.0}}
+    )
+    @ddt.unpack
+    def test_convert_new_atomic_actions(self, atomic_actions, expected):
+        atomic_wrapper = utils.WrapperForAtomicActions(atomic_actions)
+        self.assertEqual(expected,
+                         atomic_wrapper._convert_new_atomic_actions(
+                             atomic_actions))
