@@ -16,10 +16,9 @@
 import difflib
 import os
 
-from oslo_utils import encodeutils
-
 import rally
 from rally.cli import cliutils
+from rally.utils import encodeutils
 from tests.unit import test
 
 RES_PATH = os.path.join(os.path.dirname(rally.__file__), os.pardir, "etc")
@@ -32,7 +31,7 @@ class BashCompletionTestCase(test.TestCase):
         new = cliutils._generate_bash_completion_script().splitlines()
         if old != new:
             for line in difflib.unified_diff(old, new):
-                print (line)
+                print(line)
             new_filename = "/tmp/rally.bash.new"
             with open(new_filename, "wb") as new_file:
                 new_file.write(encodeutils.safe_encode("\n".join(new)))
